# encoding: utf-8

import time
from timeit import default_timer

import allure
import pytest

from aiotest.shape import LoadUserShape


# 测试用的负载形状类（在测试文件中定义，避免导入问题）
class TestLoadShape(LoadUserShape):
    """测试用的负载形状实现"""

    def tick(self):
        return (10, 1.0)


class TestLoadShapeReturnNone(LoadUserShape):
    """测试返回None的负载形状实现"""

    def tick(self):
        return None


@allure.feature("LoadUserShape")
class TestLoadUserShape:
    """LoadUserShape 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试负载形状初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization(self):
        """测试 LoadUserShape 初始化时正确设置开始时间"""
        before_init = default_timer()
        shape = TestLoadShape()
        after_init = default_timer()

        assert hasattr(shape, 'start_time')
        assert isinstance(shape.start_time, float)
        # 验证开始时间在初始化前后之间
        assert before_init <= shape.start_time <= after_init

    @allure.story("时间控制")
    @allure.title("测试重置时间功能")
    @allure.severity(allure.severity_level.NORMAL)
    def test_reset_time(self):
        """测试 reset_time 方法正确重置开始时间"""
        shape = TestLoadShape()
        original_start_time = shape.start_time

        # 等待一小段时间
        time.sleep(0.1)

        # 记录重置前的时间
        before_reset = default_timer()
        # 重置时间
        shape.reset_time()
        after_reset = default_timer()

        # 验证开始时间已更新
        assert shape.start_time > original_start_time
        # 验证新的开始时间在重置前后之间
        assert before_reset <= shape.start_time <= after_reset

    @allure.story("时间控制")
    @allure.title("测试获取运行时长")
    @allure.severity(allure.severity_level.NORMAL)
    def test_get_run_time(self):
        """测试 get_run_time 方法正确计算运行时长"""
        shape = TestLoadShape()

        # 获取初始运行时长
        initial_run_time = shape.get_run_time()
        assert initial_run_time >= 0
        assert initial_run_time < 0.1

        # 等待一小段时间
        time.sleep(0.2)

        # 再次获取运行时长
        run_time = shape.get_run_time()
        assert run_time >= 0.15  # 应该至少等待了0.15秒
        assert run_time < 0.5    # 但不超过0.5秒

    @allure.story("时间控制")
    @allure.title("测试重置时间后运行时长")
    @allure.severity(allure.severity_level.NORMAL)
    def test_get_run_time_after_reset(self):
        """测试重置时间后运行时长从0开始计算"""
        shape = TestLoadShape()

        # 等待一小段时间
        time.sleep(0.1)

        # 重置时间
        shape.reset_time()

        # 获取运行时长
        run_time = shape.get_run_time()
        assert run_time >= 0
        assert run_time < 0.1  # 重置后应该接近0

    @allure.story("抽象方法")
    @allure.title("测试 tick 方法返回用户数和速率")
    @allure.severity(allure.severity_level.NORMAL)
    def test_tick_returns_tuple(self):
        """测试 tick 方法返回正确的用户数和速率元组"""
        shape = TestLoadShape()
        result = shape.tick()

        assert result is not None
        assert isinstance(result, tuple)
        assert len(result) == 2

        user_count, rate = result
        assert isinstance(user_count, int)
        assert isinstance(rate, float)
        assert user_count == 10
        assert rate == 1.0

    @allure.story("抽象方法")
    @allure.title("测试 tick 方法返回 None")
    @allure.severity(allure.severity_level.NORMAL)
    def test_tick_returns_none(self):
        """测试 tick 方法可以返回 None 来停止测试"""
        shape = TestLoadShapeReturnNone()
        result = shape.tick()

        assert result is None

    @allure.story("边界情况")
    @allure.title("测试多次重置时间")
    @allure.severity(allure.severity_level.NORMAL)
    def test_multiple_reset_time(self):
        """测试多次重置时间的稳定性"""
        shape = TestLoadShape()

        for _ in range(3):
            original_time = shape.start_time
            time.sleep(0.05)
            before_reset = default_timer()
            shape.reset_time()
            after_reset = default_timer()
            assert shape.start_time > original_time
            assert before_reset <= shape.start_time <= after_reset

    @allure.story("边界情况")
    @allure.title("测试零用户和零速率")
    @allure.severity(allure.severity_level.NORMAL)
    def test_tick_zero_values(self):
        """测试 tick 方法处理零值的情况"""
        class ZeroLoadShape(LoadUserShape):
            def tick(self):
                return (0, 0.0)

        shape = ZeroLoadShape()
        result = shape.tick()

        assert result == (0, 0.0)

    @allure.story("边界情况")
    @allure.title("测试大数值")
    @allure.severity(allure.severity_level.NORMAL)
    def test_tick_large_values(self):
        """测试 tick 方法处理大数值的情况"""
        class LargeLoadShape(LoadUserShape):
            def tick(self):
                return (1000000, 9999.99)

        shape = LargeLoadShape()
        result = shape.tick()

        assert result == (1000000, 9999.99)

    @allure.story("暂停/恢复")
    @allure.title("测试暂停功能")
    @allure.severity(allure.severity_level.NORMAL)
    def test_pause(self):
        """测试暂停功能"""
        shape = TestLoadShape()

        # 暂停
        shape.pause()

        # 验证暂停状态
        assert shape.is_paused is True
        assert shape.paused_time > 0

    @allure.story("暂停/恢复")
    @allure.title("测试恢复功能")
    @allure.severity(allure.severity_level.NORMAL)
    def test_resume(self):
        """测试恢复功能"""
        shape = TestLoadShape()

        # 暂停
        shape.pause()
        assert shape.is_paused is True

        # 等待一小段时间
        time.sleep(0.1)

        # 恢复
        shape.resume()

        # 验证恢复状态
        assert shape.is_paused is False
        assert shape.pause_time > 0

    @allure.story("暂停/恢复")
    @allure.title("测试暂停期间的运行时间计算")
    @allure.severity(allure.severity_level.NORMAL)
    def test_run_time_with_pause(self):
        """测试暂停期间的运行时间计算"""
        shape = TestLoadShape()

        # 等待一小段时间
        time.sleep(0.1)

        # 记录暂停前的运行时间
        run_time_before_pause = shape.get_run_time()

        # 暂停
        shape.pause()

        # 等待一小段时间（模拟暂停期间）
        time.sleep(0.1)

        # 记录暂停期间的运行时间（应该与暂停前相同）
        run_time_during_pause = shape.get_run_time()

        # 恢复
        shape.resume()

        # 记录恢复后的运行时间
        run_time_after_resume = shape.get_run_time()

        # 验证运行时间计算正确（使用近似比较）
        assert abs(run_time_during_pause - run_time_before_pause) < 0.001
        assert run_time_after_resume > run_time_before_pause

    @allure.story("暂停/恢复")
    @allure.title("测试多次暂停/恢复")
    @allure.severity(allure.severity_level.NORMAL)
    def test_multiple_pause_resume(self):
        """测试多次暂停/恢复"""
        shape = TestLoadShape()

        # 第一次暂停
        shape.pause()
        assert shape.is_paused is True

        # 第一次恢复
        shape.resume()
        assert shape.is_paused is False
        assert shape.pause_time > 0

        # 第二次暂停
        shape.pause()
        assert shape.is_paused is True

        # 第二次恢复
        shape.resume()
        assert shape.is_paused is False
        assert shape.pause_time > 0

    @allure.story("暂停/恢复")
    @allure.title("测试重复暂停")
    @allure.severity(allure.severity_level.NORMAL)
    def test_double_pause(self):
        """测试重复暂停"""
        shape = TestLoadShape()

        # 第一次暂停
        shape.pause()
        paused_time_first = shape.paused_time
        assert shape.is_paused is True

        # 第二次暂停（应该没有效果）
        shape.pause()
        paused_time_second = shape.paused_time
        assert shape.is_paused is True
        assert paused_time_first == paused_time_second

    @allure.story("暂停/恢复")
    @allure.title("测试重复恢复")
    @allure.severity(allure.severity_level.NORMAL)
    def test_double_resume(self):
        """测试重复恢复"""
        shape = TestLoadShape()

        # 恢复（应该没有效果，因为没有暂停）
        shape.resume()
        assert shape.is_paused is False
        assert shape.pause_time == 0.0

        # 暂停
        shape.pause()
        assert shape.is_paused is True

        # 恢复
        shape.resume()
        assert shape.is_paused is False
        pause_time_first = shape.pause_time

        # 再次恢复（应该没有效果）
        shape.resume()
        assert shape.is_paused is False
        pause_time_second = shape.pause_time
        assert pause_time_first == pause_time_second

    @allure.story("暂停/恢复")
    @allure.title("测试重置时间时的暂停状态")
    @allure.severity(allure.severity_level.NORMAL)
    def test_reset_time_with_pause(self):
        """测试重置时间时的暂停状态"""
        shape = TestLoadShape()

        # 暂停
        shape.pause()
        assert shape.is_paused is True

        # 重置时间
        shape.reset_time()

        # 验证重置后的状态
        assert shape.is_paused is False
        assert shape.paused_time == 0.0
        assert shape.pause_time == 0.0


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8
"""
AioTest自定义异常模块

定义了框架中使用的所有自定义异常类型，提供清晰的错误分类和详细的错误信息。
"""


class AioTestError(Exception):
    """AioTest框架基础异常类"""

    def __init__(self, message: str, error_code: str = None,
                 context: dict = None):
        """
        初始化异常

        Args:
            message: 错误信息
            error_code: 错误代码，用于程序化处理
            context: 错误上下文信息，包含相关的状态数据
        """
        super().__init__(message)
        self.message = message
        self.error_code = error_code or self.__class__.__name__
        self.context = context or {}

    def __str__(self):
        return f"[{self.error_code}] {self.message}"


class RunnerError(AioTestError):
    """运行器异常，用于测试运行过程中的错误"""
    pass


class InvalidUserCountError(RunnerError):
    """无效用户数量异常"""

    def __init__(self, message: str, user_count: int = None):
        context = {'user_count': user_count} if user_count is not None else {}
        super().__init__(message, "INVALID_USER_COUNT", context)
        self.user_count = user_count


class InvalidRateError(RunnerError):
    """无效速率异常"""

    def __init__(self, message: str, rate: float = None):
        context = {'rate': rate} if rate is not None else {}
        super().__init__(message, "INVALID_RATE", context)
        self.rate = rate


# 异常映射表，用于错误码到异常类的转换
EXCEPTION_MAP = {
    "INVALID_USER_COUNT": InvalidUserCountError,
    "INVALID_RATE": InvalidRateError,
}


def create_exception(error_code: str, message: str,
                     context: dict = None) -> AioTestError:
    """
    根据错误码创建对应的异常实例

    Args:
        error_code: 错误代码
        message: 错误信息
        context: 错误上下文

    Returns:
        对应的异常实例

    Raises:
        ValueError: 如果错误码无效
    """
    exception_class = EXCEPTION_MAP.get(error_code, AioTestError)

    # 对于 AioTestError 或其他没有特殊参数的异常
    if exception_class == AioTestError:
        return exception_class(message, error_code, context)

    # 对于有特殊参数的异常类，从 context 中提取参数
    context = context or {}
    if exception_class == InvalidUserCountError:
        return exception_class(message, user_count=context.get('user_count'))
    elif exception_class == InvalidRateError:
        return exception_class(message, rate=context.get('rate'))
    else:
        return exception_class(message, error_code, context)



# encoding: utf-8

import asyncio
import logging

import allure
import pytest

from aiotest.exception import InvalidRateError, InvalidUserCountError
from aiotest.metrics import MetricsCollector
from aiotest.runner_factory import (NODE_TYPE_LOCAL, NODE_TYPE_MASTER,
                                    NODE_TYPE_WORKER, BaseRunner,
                                    EventHandlerRegistry, RunnerFactory,
                                    on_startup_completed,
                                    on_worker_request_metrics,
                                    validate_load_params, validate_params)
from aiotest.shape import LoadUserShape
from aiotest.state_manager import RunnerState
from aiotest.users import User


# 测试用的用户类
class TestUser(User):
    """测试用的用户类"""

    async def test_task(self):
        pass


# 测试用的负载形状类
class TestLoadShape(LoadUserShape):
    """测试用的负载形状类"""

    def tick(self):
        return (10, 1.0)


@allure.feature("参数验证")
class TestValidateLoadParams:
    """验证负载参数函数的测试用例"""

    @allure.story("有效参数")
    @allure.title("测试有效的用户数和速率")
    @allure.severity(allure.severity_level.NORMAL)
    @pytest.mark.parametrize("user_count,rate", [
        (1, 1.0),
        (10, 5.0),
        (100, 50.0),
        (100, 100.0),
    ])
    def test_valid_params(self, user_count, rate):
        """测试有效的用户数和速率组合"""
        validate_load_params(user_count, rate)

    @allure.story("无效参数")
    @allure.title("测试无效的用户数")
    @allure.severity(allure.severity_level.NORMAL)
    @pytest.mark.parametrize("user_count", [0, -1, -10])
    def test_invalid_user_count(self, user_count):
        """测试无效的用户数应该抛出 InvalidUserCountError"""
        with pytest.raises(InvalidUserCountError):
            validate_load_params(user_count, 1.0)

    @allure.story("无效参数")
    @allure.title("测试非整数用户数")
    @allure.severity(allure.severity_level.NORMAL)
    def test_non_integer_user_count(self):
        """测试非整数用户数应该抛出 InvalidUserCountError"""
        with pytest.raises(InvalidUserCountError):
            validate_load_params(10.5, 1.0)

    @allure.story("无效参数")
    @allure.title("测试无效的速率")
    @allure.severity(allure.severity_level.NORMAL)
    @pytest.mark.parametrize("rate", [0, -1, 11])
    def test_invalid_rate(self, rate):
        """测试无效的速率应该抛出 InvalidRateError"""
        with pytest.raises(InvalidRateError):
            validate_load_params(10, rate)


@allure.feature("参数验证装饰器")
class TestValidateParams:
    """参数验证装饰器的测试用例"""

    @allure.story("装饰器功能")
    @allure.title("测试装饰器验证参数")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_validate_params_decorator(self):
        """测试装饰器正确验证参数"""
        class TestClass:
            @validate_params
            async def test_method(self, user_count, rate, extra_arg):
                return (user_count, rate, extra_arg)

        obj = TestClass()

        result = await obj.test_method(10, 5.0, "extra")
        assert result == (10, 5.0, "extra")

        with pytest.raises(InvalidUserCountError):
            await obj.test_method(0, 5.0, "extra")


@allure.feature("事件处理器注册")
class TestEventHandlerRegistry:
    """事件处理器注册中心的测试用例"""

    @allure.story("注册功能")
    @allure.title("测试事件处理器注册")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_register_handlers(self):
        """测试事件处理器注册功能"""
        registry = EventHandlerRegistry()

        class MockRunner:
            node = NODE_TYPE_LOCAL

        runner = MockRunner()
        await registry.register_handlers(runner)

        assert id(runner) in registry._registered_handlers

    @allure.story("重复注册")
    @allure.title("测试重复注册处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_duplicate_registration(self):
        """测试重复注册不会出错"""
        registry = EventHandlerRegistry()

        class MockRunner:
            node = NODE_TYPE_LOCAL

        runner = MockRunner()
        await registry.register_handlers(runner)
        await registry.register_handlers(runner)

        assert len(registry._registered_handlers) == 1


@allure.feature("基础运行器")
class TestBaseRunner:
    """基础运行器的测试用例"""

    @allure.story("初始化")
    @allure.title("测试基础运行器初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_base_runner_initialization(self, base_config):
        """测试基础运行器正确初始化"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        assert runner.user_types == [TestUser]
        assert runner.load_shape == load_shape
        assert runner.config == base_config
        assert runner.node is None
        assert runner.cpu_usage == 0

    @allure.story("属性延迟加载")
    @allure.title("测试组件延迟初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_lazy_initialization(self, base_config):
        """测试组件的延迟初始化"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        assert runner._user_manager is None
        assert runner._state_manager is None
        assert runner._task_manager is None
        assert runner._load_shape_manager is None

        assert runner.user_manager is not None
        assert runner.state_manager is not None
        assert runner.task_manager is not None

    @allure.story("属性访问")
    @allure.title("测试活跃用户数量属性")
    @allure.severity(allure.severity_level.NORMAL)
    def test_active_user_count_property(self, base_config):
        """测试活跃用户数量属性"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        count = runner.active_user_count
        assert isinstance(count, int)
        assert count >= 0

    @allure.story("属性访问")
    @allure.title("测试状态属性")
    @allure.severity(allure.severity_level.NORMAL)
    def test_state_property(self, base_config):
        """测试状态属性"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        state = runner.state
        assert state is not None


@allure.feature("事件处理器")
class TestEventHandlers:
    """事件处理函数的测试用例"""

    @allure.story("启动完成")
    @allure.title("测试本地运行器启动完成处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_on_startup_completed_local(self):
        """测试本地运行器的启动完成事件处理"""
        from aiotest.state_manager import StateManager

        class MockRunner:
            node = NODE_TYPE_LOCAL
            active_user_count = 10

            def __init__(self):
                self.state_manager = StateManager()

        runner = MockRunner()
        # 先转换到 STARTING 状态，然后才能转换到 RUNNING
        await runner.state_manager.transition_state(RunnerState.STARTING)

        await on_startup_completed(
            node_type=NODE_TYPE_LOCAL,
            runner=runner
        )

    @allure.story("启动完成")
    @allure.title("测试Master启动完成处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_on_startup_completed_master(self):
        """测试Master运行器的启动完成事件处理"""
        from aiotest.state_manager import StateManager

        class MockRunner:
            node = NODE_TYPE_MASTER
            _startup_completion_tracker = {
                "startup_data": {"user_count": 100},
                "completed_workers": {"worker1", "worker2"},
                "expected_workers": 2
            }

            def __init__(self):
                self.state_manager = StateManager()

        runner = MockRunner()
        # 先转换到 STARTING 状态，然后才能转换到 RUNNING
        await runner.state_manager.transition_state(RunnerState.STARTING)

        await on_startup_completed(
            node_type=NODE_TYPE_MASTER,
            runner=runner
        )

    @allure.story("指标处理")
    @allure.title("测试Worker请求指标处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_on_worker_request_metrics(self):
        """测试Worker请求指标处理"""
        class MockRunner:
            node = NODE_TYPE_MASTER

            def __init__(self):
                self.metrics_collector = MetricsCollector()

        runner = MockRunner()

        metrics_data = {
            "request_id": "test-123",
            "method": "GET",
            "endpoint": "/test",
            "status_code": 200,
            "duration": 0.1,
            "response_size": 100
        }

        await on_worker_request_metrics(
            node_type=NODE_TYPE_MASTER,
            runner=runner,
            data=metrics_data
        )

    @allure.story("指标处理")
    @allure.title("测试非Master节点忽略指标")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_on_worker_request_metrics_non_master(self):
        """测试非Master节点忽略Worker请求指标"""
        class MockRunner:
            node = NODE_TYPE_WORKER

        runner = MockRunner()

        await on_worker_request_metrics(
            node_type=NODE_TYPE_WORKER,
            runner=runner,
            data={"test": "data"}
        )


@allure.feature("运行器工厂")
class TestRunnerFactory:
    """运行器工厂的测试用例"""

    @allure.story("创建运行器")
    @allure.title("测试创建本地运行器")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_create_local_runner(self, base_config):
        """测试创建本地运行器"""
        load_shape = TestLoadShape()
        config = type('Config', (), {
            'host': 'http://localhost:8080',
            'timeout': 30,
            'prometheus_port': 9090,
            'metrics_collection_interval': 5.0,
            'metrics_batch_size': 100,
            'metrics_flush_interval': 1.0,
            'metrics_buffer_size': 10000
        })()

        runner = await RunnerFactory.create(
            runner_type="local",
            user_types=[TestUser],
            load_shape=load_shape,
            config=config
        )

        assert runner is not None
        assert runner.node == NODE_TYPE_LOCAL

    @allure.story("创建运行器")
    @allure.title("测试创建无效运行器类型")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_create_invalid_runner_type(self, base_config):
        """测试创建无效的运行器类型"""
        load_shape = TestLoadShape()

        with pytest.raises(ValueError):
            await RunnerFactory.create(
                runner_type="invalid",
                user_types=[TestUser],
                load_shape=load_shape,
                config=base_config
            )

    @allure.story("创建运行器")
    @allure.title("测试创建Master运行器")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_create_master_runner(self, redis_client):
        """测试创建Master运行器"""
        load_shape = TestLoadShape()
        config = type('Config', (), {
            'host': 'http://localhost:8080',
            'timeout': 30,
            'prometheus_port': 9090,
            'metrics_collection_interval': 5.0,
            'metrics_batch_size': 100,
            'metrics_flush_interval': 1.0,
            'metrics_buffer_size': 10000
        })()

        runner = await RunnerFactory.create(
            runner_type="master",
            user_types=[TestUser],
            load_shape=load_shape,
            config=config,
            redis_client=redis_client
        )

        assert runner is not None
        assert runner.node == NODE_TYPE_MASTER

        # 清理
        await runner.quit()

    @allure.story("创建运行器")
    @allure.title("测试创建Worker运行器")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_create_worker_runner(self, redis_client):
        """测试创建Worker运行器"""
        load_shape = TestLoadShape()
        config = type('Config', (), {
            'host': 'http://localhost:8080',
            'timeout': 30,
            'prometheus_port': 9090,
            'metrics_collection_interval': 5.0,
            'metrics_batch_size': 100,
            'metrics_flush_interval': 1.0,
            'metrics_buffer_size': 10000
        })()

        runner = await RunnerFactory.create(
            runner_type="worker",
            user_types=[TestUser],
            load_shape=load_shape,
            config=config,
            redis_client=redis_client
        )

        assert runner is not None
        assert runner.node == NODE_TYPE_WORKER

        # 清理
        await runner.quit()


@allure.feature("基础运行器异步方法")
class TestBaseRunnerAsyncMethods:
    """基础运行器异步方法的测试用例"""

    @allure.story("启动测试")
    @allure.title("测试启动测试")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_start(self, base_config):
        """测试启动测试"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 启动测试
        await runner.start()

    @allure.story("启动测试")
    @allure.title("测试启动测试无负载形状")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_start_without_load_shape(self, base_config):
        """测试没有负载形状时启动测试"""
        runner = BaseRunner([TestUser], None, base_config)

        with pytest.raises(ValueError):
            await runner.start()

    @allure.story("运行测试")
    @allure.title("测试运行直到完成")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_run_until_complete(self, base_config):
        """测试运行直到完成"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 启动测试
        await runner.start()

        # 等待一小段时间
        await asyncio.sleep(0.1)

        # 退出运行器以停止任务
        await runner.quit()

        # 运行直到完成（应该在任务完成后返回）
        await runner.run_until_complete()

    @allure.story("退出测试")
    @allure.title("测试退出运行器")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_quit(self, base_config):
        """测试退出运行器"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 启动测试
        await runner.start()

        # 等待一小段时间
        await asyncio.sleep(0.1)

        # 退出运行器
        await runner.quit()

        # 验证状态
        assert runner.state_manager.is_in_quit_state()

    @allure.story("退出测试")
    @allure.title("测试重复退出")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_quit_twice(self, base_config):
        """测试重复退出不会出错"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 启动测试
        await runner.start()

        # 等待一小段时间
        await asyncio.sleep(0.1)

        # 第一次退出
        await runner.quit()

        # 第二次退出（应该不执行任何操作）
        await runner.quit()

    @allure.story("应用负载")
    @allure.title("测试应用负载")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_apply_load(self, base_config):
        """测试应用负载配置"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 应用负载
        await runner.apply_load(user_count=5, rate=2.0)

        # 验证状态
        assert runner.state == RunnerState.RUNNING

    @allure.story("应用负载")
    @allure.title("测试应用负载无效参数")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_apply_load_invalid_params(self, base_config):
        """测试应用负载时无效参数"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 测试无效的用户数
        with pytest.raises(InvalidUserCountError):
            await runner.apply_load(user_count=0, rate=1.0)

        # 测试无效的速率
        with pytest.raises(InvalidRateError):
            await runner.apply_load(user_count=10, rate=0.0)

    @allure.story("停止测试")
    @allure.title("测试停止测试")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_stop(self, base_config):
        """测试停止测试"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 应用负载
        await runner.apply_load(user_count=5, rate=2.0)

        # 停止测试
        with pytest.raises(NotImplementedError):
            await runner.stop()

    @allure.story("CPU指标收集")
    @allure.title("测试收集CPU指标")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_collect_cpu_metrics(self, base_config):
        """测试收集CPU指标"""
        load_shape = TestLoadShape()
        runner = BaseRunner([TestUser], load_shape, base_config)

        # 启动测试
        await runner.start()

        # 等待一小段时间
        await asyncio.sleep(0.1)

        # 退出运行器
        await runner.quit()

        # 验证CPU使用率被收集
        assert isinstance(runner.cpu_usage, (int, float))


@allure.feature("高速率警告")
class TestHighRateWarning:
    """高速率警告的测试用例"""

    @allure.story("警告日志")
    @allure.title("测试高速率警告")
    @allure.severity(allure.severity_level.NORMAL)
    def test_high_rate_warning(self, base_config):
        """测试高速率警告日志 - 验证函数不抛出异常"""
        # 调用验证函数，触发高速率警告（应该正常执行，不抛出异常）
        validate_load_params(user_count=200, rate=150.0)
        # 如果函数正常执行，说明警告已记录（日志输出到控制台）


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8

import allure
import pytest

from aiotest.state_manager import RunnerState, StateManager


@allure.feature("StateManager")
class TestStateManager:
    """StateManager 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试 StateManager 初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization(self):
        """测试 StateManager 初始化时正确设置状态"""
        manager = StateManager()
        assert manager.get_current_state() == RunnerState.READY
        assert manager.is_in_quit_state() is False

    @allure.story("状态转换")
    @allure.title("测试合法状态转换")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_valid_state_transitions(self):
        """测试合法的状态转换"""
        manager = StateManager()

        # READY -> STARTING
        await manager.transition_state(RunnerState.STARTING)
        assert manager.get_current_state() == RunnerState.STARTING

        # STARTING -> RUNNING
        await manager.transition_state(RunnerState.RUNNING)
        assert manager.get_current_state() == RunnerState.RUNNING

        # RUNNING -> STOPPING
        await manager.transition_state(RunnerState.STOPPING)
        assert manager.get_current_state() == RunnerState.STOPPING

        # STOPPING -> READY
        await manager.transition_state(RunnerState.READY)
        assert manager.get_current_state() == RunnerState.READY

    @allure.story("状态转换")
    @allure.title("测试非法状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_invalid_state_transitions(self):
        """测试非法的状态转换"""
        manager = StateManager()

        # READY -> RUNNING (非法转换)
        with pytest.raises(RuntimeError):
            await manager.transition_state(RunnerState.RUNNING)

        # READY -> STOPPING (非法转换)
        with pytest.raises(RuntimeError):
            await manager.transition_state(RunnerState.STOPPING)

    @allure.story("状态转换")
    @allure.title("测试紧急停止状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_emergency_stop_transition(self):
        """测试紧急停止状态转换"""
        manager = StateManager()

        # 启动到运行状态
        await manager.transition_state(RunnerState.STARTING)
        await manager.transition_state(RunnerState.RUNNING)

        # RUNNING -> QUITTING (紧急停止)
        await manager.transition_state(RunnerState.QUITTING)
        assert manager.get_current_state() == RunnerState.QUITTING

    @allure.story("状态转换")
    @allure.title("测试相同状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_same_state_transition(self):
        """测试相同状态的转换（应该无操作）"""
        manager = StateManager()

        # 初始状态是 READY
        initial_state = manager.get_current_state()

        # 再次转换到 READY
        await manager.transition_state(RunnerState.READY)

        # 状态应该保持不变
        assert manager.get_current_state() == initial_state

    @allure.story("状态查询")
    @allure.title("测试是否可以启动")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_can_start(self):
        """测试 can_start 方法"""
        manager = StateManager()

        # READY 状态应该可以启动
        assert manager.can_start() is True

        # 启动后应该不能启动
        await manager.transition_state(RunnerState.STARTING)
        assert manager.can_start() is False

        # 运行中应该不能启动
        await manager.transition_state(RunnerState.RUNNING)
        assert manager.can_start() is False

    @allure.story("状态查询")
    @allure.title("测试是否正在运行")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_is_running(self):
        """测试 is_running 方法"""
        manager = StateManager()

        # READY 状态应该不在运行
        assert manager.is_running() is False

        # 启动中应该不在运行
        await manager.transition_state(RunnerState.STARTING)
        assert manager.is_running() is False

        # 运行中应该在运行
        await manager.transition_state(RunnerState.RUNNING)
        assert manager.is_running() is True

    @allure.story("状态查询")
    @allure.title("测试是否可以停止")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_can_stop(self):
        """测试 can_stop 方法"""
        manager = StateManager()

        # READY 状态应该不能停止
        assert manager.can_stop() is False

        # 启动中应该可以停止
        await manager.transition_state(RunnerState.STARTING)
        assert manager.can_stop() is True

        # 运行中应该可以停止
        await manager.transition_state(RunnerState.RUNNING)
        assert manager.can_stop() is True

    @allure.story("退出状态")
    @allure.title("测试设置退出状态")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_set_quit_state(self):
        """测试设置退出状态"""
        manager = StateManager()

        # 初始状态
        assert manager.is_in_quit_state() is False

        # 设置退出状态
        manager.set_quit_state()
        assert manager.is_in_quit_state() is True

        # 退出状态下状态转换应该无操作
        initial_state = manager.get_current_state()
        await manager.transition_state(RunnerState.STARTING)
        assert manager.get_current_state() == initial_state

    @allure.story("状态转换")
    @allure.title("测试 MISSING 状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_missing_state_transition(self):
        """测试 MISSING 状态的转换"""
        # 注意：这里需要先将状态设置为 MISSING，
        # 但由于 transition_state 方法的限制，我们需要通过直接修改状态来测试
        manager = StateManager()

        # 直接修改状态为 MISSING（仅用于测试）
        manager.state = RunnerState.MISSING

        # MISSING 状态只能转换到 QUITTING
        await manager.transition_state(RunnerState.QUITTING)
        assert manager.get_current_state() == RunnerState.QUITTING

    @allure.story("状态转换")
    @allure.title("测试 STOPPING 状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_stopping_state_transition(self):
        """测试 STOPPING 状态的转换"""
        manager = StateManager()

        # 启动到运行状态
        await manager.transition_state(RunnerState.STARTING)
        await manager.transition_state(RunnerState.RUNNING)

        # 转换到 STOPPING
        await manager.transition_state(RunnerState.STOPPING)
        assert manager.get_current_state() == RunnerState.STOPPING

        # STOPPING -> READY
        await manager.transition_state(RunnerState.READY)
        assert manager.get_current_state() == RunnerState.READY

    @allure.story("状态转换")
    @allure.title("测试 QUITTING 状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_quitting_state_transition(self):
        """测试 QUITTING 状态的转换"""
        manager = StateManager()

        # 转换到 QUITTING
        await manager.transition_state(RunnerState.QUITTING)
        assert manager.get_current_state() == RunnerState.QUITTING

        # QUITTING 状态不能转换到其他状态
        with pytest.raises(RuntimeError):
            await manager.transition_state(RunnerState.READY)

    @allure.story("状态转换")
    @allure.title("测试 PAUSED 状态转换")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_paused_state_transitions(self):
        """测试 PAUSED 状态的转换"""
        manager = StateManager()

        # 启动到运行状态
        await manager.transition_state(RunnerState.STARTING)
        await manager.transition_state(RunnerState.RUNNING)

        # RUNNING -> PAUSED
        await manager.transition_state(RunnerState.PAUSED)
        assert manager.get_current_state() == RunnerState.PAUSED

        # PAUSED -> RUNNING
        await manager.transition_state(RunnerState.RUNNING)
        assert manager.get_current_state() == RunnerState.RUNNING

        # 再次暂停
        await manager.transition_state(RunnerState.PAUSED)
        assert manager.get_current_state() == RunnerState.PAUSED

        # PAUSED -> STOPPING
        await manager.transition_state(RunnerState.STOPPING)
        assert manager.get_current_state() == RunnerState.STOPPING

        # STOPPING -> READY
        await manager.transition_state(RunnerState.READY)
        assert manager.get_current_state() == RunnerState.READY

    @allure.story("状态查询")
    @allure.title("测试是否可以暂停")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_can_pause(self):
        """测试 can_pause 方法"""
        manager = StateManager()

        # READY 状态应该不能暂停
        assert manager.can_pause() is False

        # 启动中应该不能暂停
        await manager.transition_state(RunnerState.STARTING)
        assert manager.can_pause() is False

        # 运行中应该可以暂停
        await manager.transition_state(RunnerState.RUNNING)
        assert manager.can_pause() is True

        # 暂停状态应该不能暂停
        await manager.transition_state(RunnerState.PAUSED)
        assert manager.can_pause() is False

    @allure.story("状态查询")
    @allure.title("测试是否可以恢复")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_can_resume(self):
        """测试 can_resume 方法"""
        manager = StateManager()

        # READY 状态应该不能恢复
        assert manager.can_resume() is False

        # 启动中应该不能恢复
        await manager.transition_state(RunnerState.STARTING)
        assert manager.can_resume() is False

        # 运行中应该不能恢复
        await manager.transition_state(RunnerState.RUNNING)
        assert manager.can_resume() is False

        # 暂停状态应该可以恢复
        await manager.transition_state(RunnerState.PAUSED)
        assert manager.can_resume() is True

    @allure.story("状态转换")
    @allure.title("测试非法暂停状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_invalid_pause_transitions(self):
        """测试非法的暂停状态转换"""
        manager = StateManager()

        # READY -> PAUSED (非法转换)
        with pytest.raises(RuntimeError):
            await manager.transition_state(RunnerState.PAUSED)

        # STARTING -> PAUSED (非法转换)
        await manager.transition_state(RunnerState.STARTING)
        with pytest.raises(RuntimeError):
            await manager.transition_state(RunnerState.PAUSED)

    @allure.story("状态转换")
    @allure.title("测试非法恢复状态转换")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_invalid_resume_transitions(self):
        """测试非法的恢复状态转换"""
        manager = StateManager()

        # READY -> RUNNING (非法转换，应该先到 STARTING)
        with pytest.raises(RuntimeError):
            await manager.transition_state(RunnerState.RUNNING)

        # 启动到运行状态
        await manager.transition_state(RunnerState.STARTING)
        await manager.transition_state(RunnerState.RUNNING)

        # 启动到暂停状态
        await manager.transition_state(RunnerState.PAUSED)

        # PAUSED -> PAUSED (非法转换，应该无操作)
        # 由于 transition_state 方法会检查相同状态并直接返回，所以不会抛出异常
        await manager.transition_state(RunnerState.PAUSED)
        assert manager.get_current_state() == RunnerState.PAUSED


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8

import allure
import pytest

from aiotest.exception import (AioTestError, InvalidRateError,
                               InvalidUserCountError, RunnerError,
                               create_exception)


@allure.feature("异常类测试")
class TestAioTestError:
    """测试 AioTestError 基础异常类"""

    @allure.story("初始化")
    @allure.title("测试基本初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init_basic(self):
        """测试基本初始化"""
        error = AioTestError("Test error")
        assert error.message == "Test error"
        assert error.error_code == "AioTestError"
        assert error.context == {}

    @allure.story("初始化")
    @allure.title("测试带错误码初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init_with_error_code(self):
        """测试带错误码初始化"""
        error = AioTestError("Test error", "TEST_ERROR")
        assert error.message == "Test error"
        assert error.error_code == "TEST_ERROR"

    @allure.story("初始化")
    @allure.title("测试带上下文初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init_with_context(self):
        """测试带上下文初始化"""
        context = {"key": "value"}
        error = AioTestError("Test error", "TEST_ERROR", context)
        assert error.message == "Test error"
        assert error.error_code == "TEST_ERROR"
        assert error.context == context

    @allure.story("字符串表示")
    @allure.title("测试 __str__ 方法")
    @allure.severity(allure.severity_level.NORMAL)
    def test_str_method(self):
        """测试 __str__ 方法"""
        error = AioTestError("Test error", "TEST_ERROR")
        assert str(error) == "[TEST_ERROR] Test error"


@allure.feature("异常类测试")
class TestRunnerError:
    """测试 RunnerError 异常类"""

    @allure.story("初始化")
    @allure.title("测试 RunnerError 初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init(self):
        """测试 RunnerError 初始化"""
        error = RunnerError("Runner error")
        assert error.message == "Runner error"
        assert error.error_code == "RunnerError"
        assert isinstance(error, AioTestError)


@allure.feature("异常类测试")
class TestInvalidUserCountError:
    """测试 InvalidUserCountError 异常类"""

    @allure.story("初始化")
    @allure.title("测试基本初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init_basic(self):
        """测试基本初始化"""
        error = InvalidUserCountError("Invalid user count")
        assert error.message == "Invalid user count"
        assert error.error_code == "INVALID_USER_COUNT"
        assert error.user_count is None
        assert error.context == {}

    @allure.story("初始化")
    @allure.title("测试带用户数量初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init_with_user_count(self):
        """测试带用户数量初始化"""
        error = InvalidUserCountError("Invalid user count", 100)
        assert error.message == "Invalid user count"
        assert error.error_code == "INVALID_USER_COUNT"
        assert error.user_count == 100
        assert error.context == {"user_count": 100}

    @allure.story("字符串表示")
    @allure.title("测试 __str__ 方法")
    @allure.severity(allure.severity_level.NORMAL)
    def test_str_method(self):
        """测试 __str__ 方法"""
        error = InvalidUserCountError("Invalid user count", 100)
        assert str(error) == "[INVALID_USER_COUNT] Invalid user count"


@allure.feature("异常类测试")
class TestInvalidRateError:
    """测试 InvalidRateError 异常类"""

    @allure.story("初始化")
    @allure.title("测试基本初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init_basic(self):
        """测试基本初始化"""
        error = InvalidRateError("Invalid rate")
        assert error.message == "Invalid rate"
        assert error.error_code == "INVALID_RATE"
        assert error.rate is None
        assert error.context == {}

    @allure.story("初始化")
    @allure.title("测试带速率初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_init_with_rate(self):
        """测试带速率初始化"""
        error = InvalidRateError("Invalid rate", 10.5)
        assert error.message == "Invalid rate"
        assert error.error_code == "INVALID_RATE"
        assert error.rate == 10.5
        assert error.context == {"rate": 10.5}

    @allure.story("字符串表示")
    @allure.title("测试 __str__ 方法")
    @allure.severity(allure.severity_level.NORMAL)
    def test_str_method(self):
        """测试 __str__ 方法"""
        error = InvalidRateError("Invalid rate", 10.5)
        assert str(error) == "[INVALID_RATE] Invalid rate"


@allure.feature("异常工具函数测试")
class TestCreateException:
    """测试 create_exception 函数"""

    @allure.story("创建异常")
    @allure.title("测试创建基础 AioTestError")
    @allure.severity(allure.severity_level.NORMAL)
    def test_create_aio_test_error(self):
        """测试创建基础 AioTestError"""
        error = create_exception("UNKNOWN_ERROR", "Unknown error")
        assert isinstance(error, AioTestError)
        assert error.message == "Unknown error"
        assert error.error_code == "UNKNOWN_ERROR"

    @allure.story("创建异常")
    @allure.title("测试创建 InvalidUserCountError")
    @allure.severity(allure.severity_level.NORMAL)
    def test_create_invalid_user_count_error(self):
        """测试创建 InvalidUserCountError"""
        context = {"user_count": 100}
        error = create_exception(
            "INVALID_USER_COUNT",
            "Invalid user count",
            context)
        assert isinstance(error, InvalidUserCountError)
        assert error.message == "Invalid user count"
        assert error.error_code == "INVALID_USER_COUNT"
        assert error.user_count == 100

    @allure.story("创建异常")
    @allure.title("测试创建 InvalidRateError")
    @allure.severity(allure.severity_level.NORMAL)
    def test_create_invalid_rate_error(self):
        """测试创建 InvalidRateError"""
        context = {"rate": 10.5}
        error = create_exception("INVALID_RATE", "Invalid rate", context)
        assert isinstance(error, InvalidRateError)
        assert error.message == "Invalid rate"
        assert error.error_code == "INVALID_RATE"
        assert error.rate == 10.5

    @allure.story("创建异常")
    @allure.title("测试创建异常时上下文为 None")
    @allure.severity(allure.severity_level.NORMAL)
    def test_create_exception_with_none_context(self):
        """测试创建异常时上下文为 None"""
        error = create_exception(
            "INVALID_USER_COUNT",
            "Invalid user count",
            None)
        assert isinstance(error, InvalidUserCountError)
        assert error.user_count is None


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8

import asyncio

import allure
import pytest

from aiotest.user_manager import UserManager
from aiotest.users import User, weight


# 测试用的用户类
class TestUser(User):
    """测试用的用户类"""

    async def test_task(self):
        await asyncio.sleep(0.1)


@weight(2)
class WeightedUser1(User):
    """权重为2的用户类"""

    async def test_task(self):
        await asyncio.sleep(0.1)


@weight(3)
class WeightedUser2(User):
    """权重为3的用户类"""

    async def test_task(self):
        await asyncio.sleep(0.1)


@weight(1)
class WeightedUser3(User):
    """权重为1的用户类"""

    async def test_task(self):
        await asyncio.sleep(0.1)


@allure.feature("UserManager")
class TestUserManager:
    """UserManager 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试 UserManager 基本初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization(self):
        """测试 UserManager 初始化时正确设置属性"""
        user_types = [TestUser, WeightedUser1]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        assert manager.user_types == user_types
        assert manager.config == config
        assert manager.active_users == []

    @allure.story("初始化")
    @allure.title("测试 UserManager 空用户类型初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization_empty_user_types(self):
        """测试使用空用户类型列表初始化"""
        config = {"host": "http://localhost:8080"}
        manager = UserManager([], config)

        assert manager.user_types == []
        assert manager.config == config
        assert manager.active_users == []


@allure.feature("权重分配")
class TestWeightedDistribution:
    """权重分配算法的测试用例"""

    @allure.story("权重计算")
    @allure.title("测试单一用户类型权重分配")
    @allure.severity(allure.severity_level.NORMAL)
    def test_calculate_weighted_counts_single_type(self):
        """测试单一用户类型的权重分配"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        result = manager._calculate_weighted_counts(user_types, 10)
        assert result == {TestUser: 10}

    @allure.story("权重计算")
    @allure.title("测试多用户类型权重分配")
    @allure.severity(allure.severity_level.NORMAL)
    def test_calculate_weighted_counts_multiple_types(self):
        """测试多用户类型的权重分配"""
        user_types = [WeightedUser1, WeightedUser2]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        result = manager._calculate_weighted_counts(user_types, 100)
        assert TestUser not in result
        assert WeightedUser1 in result
        assert WeightedUser2 in result
        # 权重比例 2:3，期望分配 40:60
        assert 38 <= result[WeightedUser1] <= 42
        assert 58 <= result[WeightedUser2] <= 62

    @allure.story("权重计算")
    @allure.title("测试带当前数量限制的权重分配")
    @allure.severity(allure.severity_level.NORMAL)
    def test_calculate_weighted_counts_with_current_counts(self):
        """测试带当前数量限制的权重分配"""
        user_types = [WeightedUser1, WeightedUser2]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        current_counts = {WeightedUser1: 10, WeightedUser2: 15}
        result = manager._calculate_weighted_counts(
            user_types, 30, current_counts)
        assert result[WeightedUser1] <= 10
        assert result[WeightedUser2] <= 15

    @allure.story("权重计算")
    @allure.title("测试无效权重配置")
    @allure.severity(allure.severity_level.NORMAL)
    def test_calculate_weighted_counts_invalid_weight(self):
        """测试无效的权重配置"""
        class InvalidWeightUser(User):
            weight = 0  # 无效权重

        user_types = [InvalidWeightUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        with pytest.raises(ValueError):
            manager._calculate_weighted_counts(user_types, 10)

    @allure.story("用户分配")
    @allure.title("测试基于权重的用户分配")
    @allure.severity(allure.severity_level.CRITICAL)
    def test_distribute_users_by_weight(self):
        """测试基于权重的用户分配算法"""
        user_types = [WeightedUser1, WeightedUser2]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        result = manager.distribute_users_by_weight(100)
        assert len(result) == 100

        # 统计各类型的数量
        type1_count = sum(
            1 for user_type in result if user_type == WeightedUser1)
        type2_count = sum(
            1 for user_type in result if user_type == WeightedUser2)

        assert 38 <= type1_count <= 42
        assert 58 <= type2_count <= 62

    @allure.story("用户分配")
    @allure.title("测试空用户类型分配")
    @allure.severity(allure.severity_level.NORMAL)
    def test_distribute_users_by_weight_empty(self):
        """测试空用户类型列表的分配"""
        config = {"host": "http://localhost:8080"}
        manager = UserManager([], config)

        result = manager.distribute_users_by_weight(100)
        assert result == []


@allure.feature("用户管理")
class TestUserManagement:
    """用户管理方法的测试用例"""

    @allure.story("用户管理")
    @allure.title("测试启动用户")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_start_users(self):
        """测试启动用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        await manager._start_users(5, 10.0)
        assert len(manager.active_users) == 5

    @allure.story("用户管理")
    @allure.title("测试停止用户")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_stop_users(self):
        """测试停止用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 先启动用户
        await manager._start_users(5, 10.0)
        assert len(manager.active_users) == 5

        # 停止用户
        await manager._stop_users(3, 10.0)
        # 活跃用户数应该减少

    @allure.story("用户管理")
    @allure.title("测试用户管理核心方法")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_manage_users(self):
        """测试用户管理的核心方法"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 测试启动用户
        await manager.manage_users(5, 10.0, "start")
        assert len(manager.active_users) == 5

        # 测试停止用户
        await manager.manage_users(2, 10.0, "stop")

    @allure.story("用户管理")
    @allure.title("测试无效操作类型")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_manage_users_invalid_action(self):
        """测试无效的操作类型"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        with pytest.raises(ValueError):
            await manager.manage_users(5, 10.0, "invalid")

    @allure.story("用户管理")
    @allure.title("测试无效参数")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_manage_users_invalid_params(self):
        """测试无效的参数"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 测试负数用户数
        with pytest.raises(ValueError):
            await manager.manage_users(-1, 10.0, "start")

        # 测试负数速率
        with pytest.raises(ValueError):
            await manager.manage_users(5, -1.0, "start")


@allure.feature("批量执行")
class TestBatchExecute:
    """批量执行方法的测试用例"""

    @allure.story("批量执行")
    @allure.title("测试批量执行带速率控制")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_batch_execute_with_rate(self):
        """测试带速率控制的批量执行"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        items = [TestUser, TestUser, TestUser]
        executed = []

        async def mock_operation(item):
            executed.append(item)
            await asyncio.sleep(0.01)

        start_time = asyncio.get_event_loop().time()
        await manager._batch_execute(items, mock_operation, 10.0)  # 10个/秒
        end_time = asyncio.get_event_loop().time()

        assert len(executed) == 3
        # 速率控制应该导致总时间大于 0.2 秒（3个操作，每0.1秒一个）
        assert end_time - start_time >= 0.2

    @allure.story("批量执行")
    @allure.title("测试批量执行无速率限制")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_batch_execute_no_rate(self):
        """测试无速率限制的批量执行"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        items = [TestUser, TestUser, TestUser]
        executed = []

        async def mock_operation(item):
            executed.append(item)
            await asyncio.sleep(0.01)

        start_time = asyncio.get_event_loop().time()
        await manager._batch_execute(items, mock_operation, 0)  # 无限制
        end_time = asyncio.get_event_loop().time()

        assert len(executed) == 3


@allure.feature("用户操作")
class TestUserOperations:
    """用户操作方法的测试用例"""

    @allure.story("用户操作")
    @allure.title("测试创建并启动用户")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_create_and_start_user(self):
        """测试创建并启动单个用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        await manager._create_and_start_user(TestUser)
        assert len(manager.active_users) == 1
        assert isinstance(manager.active_users[0], TestUser)

    @allure.story("用户操作")
    @allure.title("测试停止用户")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_stop_user(self):
        """测试停止单个用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 创建并启动用户
        await manager._create_and_start_user(TestUser)
        user = manager.active_users[0]

        # 停止用户
        await manager._stop_user(user)


@allure.feature("用户选择")
class TestUserSelection:
    """用户选择方法的测试用例"""

    @allure.story("用户选择")
    @allure.title("测试选择要停止的用户")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_select_users_to_stop(self):
        """测试选择要停止的用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 启动多个用户
        await manager._start_users(5, 10.0)
        assert len(manager.active_users) == 5

        # 选择要停止的用户
        users_to_stop = await manager._select_users_to_stop(3)
        assert len(users_to_stop) == 3

    @allure.story("用户选择")
    @allure.title("测试选择超出活跃用户数的情况")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_select_users_to_stop_exceed(self):
        """测试选择超出活跃用户数的情况"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 启动少量用户
        await manager._start_users(3, 10.0)
        assert len(manager.active_users) == 3

        # 选择超出数量的用户
        users_to_stop = await manager._select_users_to_stop(5)
        assert len(users_to_stop) == 3


@allure.feature("状态管理")
class TestStateManagement:
    """状态管理方法的测试用例"""

    @allure.story("状态管理")
    @allure.title("测试活跃用户数量属性")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_active_user_count(self):
        """测试活跃用户数量属性"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 初始状态
        assert manager.active_user_count == 0

        # 启动用户后
        await manager._start_users(5, 10.0)
        assert manager.active_user_count >= 0

    @allure.story("状态管理")
    @allure.title("测试清理非活跃用户")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_cleanup_inactive_users(self):
        """测试清理非活跃用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 启动用户
        await manager._start_users(3, 10.0)
        initial_count = len(manager.active_users)

        # 清理非活跃用户
        manager.cleanup_inactive_users()
        # 活跃用户数应该保持不变或减少

    @allure.story("状态管理")
    @allure.title("测试停止所有用户")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_stop_all_users(self):
        """测试停止所有用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 启动用户
        await manager._start_users(5, 10.0)
        assert len(manager.active_users) == 5

        # 停止所有用户
        await manager.stop_all_users()
        assert len(manager.active_users) == 0

    @allure.story("状态管理")
    @allure.title("测试暂停所有用户")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_pause_all_users(self):
        """测试暂停所有用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 启动用户
        await manager._start_users(3, 10.0)
        assert len(manager.active_users) == 3

        # 暂停所有用户
        await manager.pause_all_users()
        # 活跃用户列表应该保持不变
        assert len(manager.active_users) == 3

    @allure.story("状态管理")
    @allure.title("测试恢复所有用户")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_resume_all_users(self):
        """测试恢复所有用户"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 启动用户
        await manager._start_users(3, 10.0)
        assert len(manager.active_users) == 3

        # 暂停所有用户
        await manager.pause_all_users()
        
        # 恢复所有用户
        await manager.resume_all_users()
        # 活跃用户列表应该保持不变
        assert len(manager.active_users) == 3

    @allure.story("状态管理")
    @allure.title("测试空用户列表的暂停操作")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_pause_all_users_empty(self):
        """测试空用户列表的暂停操作"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 空用户列表
        assert len(manager.active_users) == 0

        # 暂停所有用户（应该无操作）
        await manager.pause_all_users()
        assert len(manager.active_users) == 0

    @allure.story("状态管理")
    @allure.title("测试空用户列表的恢复操作")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_resume_all_users_empty(self):
        """测试空用户列表的恢复操作"""
        user_types = [TestUser]
        config = {"host": "http://localhost:8080"}
        manager = UserManager(user_types, config)

        # 空用户列表
        assert len(manager.active_users) == 0

        # 恢复所有用户（应该无操作）
        await manager.resume_all_users()
        assert len(manager.active_users) == 0


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8

import asyncio
from typing import List

from aiotest.logger import logger


class TaskManager:
    """任务管理器，负责后台任务的创建、管理和清理"""

    def __init__(self):
        self.background_tasks: List[asyncio.Task] = []

    def add_task(self, task: asyncio.Task) -> None:
        """添加后台任务"""
        self.background_tasks.append(task)

    def add_tasks(self, tasks: List[asyncio.Task]) -> None:
        """批量添加后台任务"""
        self.background_tasks.extend(tasks)

    def create_task(self, coro, name: str = None) -> asyncio.Task:
        """创建并添加后台任务"""
        task = asyncio.create_task(coro, name=name)
        self.add_task(task)
        return task

    async def cancel_all_tasks(self) -> None:
        """取消所有后台任务"""
        # 筛选需要取消的任务（排除当前任务）
        tasks_to_cancel: List[asyncio.Task] = [
            t for t in self.background_tasks
            if not t.done() and t != asyncio.current_task()
        ]

        for task in tasks_to_cancel:
            task.cancel()

        if tasks_to_cancel:
            # 等待所有取消操作完成
            await asyncio.gather(*tasks_to_cancel, return_exceptions=True)
            # 清理已完成的任务
            self.background_tasks = [
                t for t in self.background_tasks if not t.done()]

    def get_task_by_name(self, name: str) -> asyncio.Task:
        """根据名称获取任务"""
        for task in self.background_tasks:
            if task.get_name() == name:
                return task
        return None

    def remove_task(self, task: asyncio.Task) -> None:
        """移除任务"""
        if task in self.background_tasks:
            self.background_tasks.remove(task)

    def clear_completed_tasks(self) -> None:
        """清理已完成的任务"""
        self.background_tasks = [
            t for t in self.background_tasks if not t.done()]

    def get_active_task_count(self) -> int:
        """获取活跃任务数量"""
        return len([t for t in self.background_tasks if not t.done()])

    async def wait_for_task_completion(self, timeout: float = None) -> None:
        """等待所有任务完成"""
        if not self.background_tasks:
            return

        try:
            await asyncio.wait_for(
                asyncio.gather(*self.background_tasks, return_exceptions=True),
                timeout=timeout
            )
        except asyncio.TimeoutError:
            logger.warning("任务完成超时，已超过 %d 秒", timeout)

    async def stop_task_by_name(self, name: str) -> bool:
        """根据名称停止任务"""
        task = self.get_task_by_name(name)
        if task and not task.done():
            task.cancel()
            try:
                await task
                return True
            except asyncio.CancelledError:
                return True
        return False



# encoding: utf-8

import asyncio

import allure
import pytest
from aiohttp.test_utils import TestClient, TestServer

from aiotest import runners
from aiotest.exception import RunnerError
from aiotest.runner_factory import (NODE_TYPE_LOCAL, NODE_TYPE_MASTER,
                                    NODE_TYPE_WORKER)
from aiotest.runners import (LocalRunner, MasterRunner, WorkerNode,
                             WorkerRunner, create_prometheus_app,
                             init_metrics_collector, start_prometheus_service)
from aiotest.state_manager import RunnerState


@allure.feature("辅助函数")
class TestHelperFunctions:
    """辅助函数的测试用例"""

    @allure.story("Prometheus应用")
    @allure.title("测试创建Prometheus应用")
    @allure.severity(allure.severity_level.NORMAL)
    def test_create_prometheus_app(self):
        """测试创建Prometheus指标HTTP应用"""
        app = create_prometheus_app()
        assert app is not None
        # 验证路由是否正确添加
        routes = [route.resource.canonical for route in app.router.routes()]
        assert "/metrics" in routes

    @allure.story("指标收集器")
    @allure.title("测试初始化指标收集器")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_init_metrics_collector(self):
        """测试初始化指标收集器"""
        # 使用真实参数进行测试
        node = NODE_TYPE_LOCAL
        redis_client = None  # Local模式不需要Redis
        node_id = "test_node"
        coordinator = None  # Local模式不需要协调器

        # 测试初始化指标收集器
        collector = await init_metrics_collector(node, redis_client, node_id, coordinator)
        assert collector is not None

        # 清理资源
        await collector.stop()

    @allure.story("Prometheus服务")
    @allure.title("测试启动Prometheus服务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_start_prometheus_service(self):
        """测试启动Prometheus服务"""
        # 模拟配置
        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8000

        config = MockConfig()

        # 启动服务
        prometheus_runner, started = await start_prometheus_service(config)
        assert started is True

        # 清理资源
        await prometheus_runner.cleanup()

    @allure.story("Prometheus服务")
    @allure.title("测试Prometheus metrics handler异常处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_metrics_handler_exception(self):
        """测试Prometheus metrics handler的异常处理"""
        # 保存原始的generate_latest函数
        original_generate_latest = runners.generate_latest
        try:
            # 创建一个模拟的generate_latest函数，它会抛出异常
            def mock_generate_latest(registry):
                raise Exception("Test exception")

            # 替换runners模块中的generate_latest引用
            runners.generate_latest = mock_generate_latest

            # 创建应用
            app = create_prometheus_app()

            # 模拟请求
            server = TestServer(app)
            async with server:
                async with TestClient(server) as client:
                    # 发送请求到/metrics端点
                    response = await client.get('/metrics')

                    # 验证响应状态码为500
                    assert response.status == 500

                    # 验证响应内容
                    content = await response.text()
                    assert 'Internal Server Error' in content
        finally:
            # 恢复原始的generate_latest函数
            runners.generate_latest = original_generate_latest


@allure.feature("WorkerNode")
class TestWorkerNode:
    """WorkerNode 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试WorkerNode初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_worker_node_initialization(self):
        """测试WorkerNode初始化时正确设置属性"""
        node_id = "test_worker"
        worker_node = WorkerNode(node_id)

        assert worker_node.node_id == node_id
        assert worker_node.status == RunnerState.READY
        assert worker_node.cpu_usage == 0.0
        assert worker_node.active_users == 0
        assert worker_node.last_update == 0.0

    @allure.story("更新状态")
    @allure.title("测试从心跳数据更新节点状态")
    @allure.severity(allure.severity_level.NORMAL)
    def test_update_from_heartbeat(self):
        """测试从心跳数据更新节点状态"""
        node_id = "test_worker"
        worker_node = WorkerNode(node_id)

        # 心跳数据
        heartbeat_data = {
            "cpu_percent": 50.5,
            "active_users": 100,
            "status": "running"
        }

        # 更新状态
        worker_node.update_from_heartbeat(heartbeat_data)

        assert worker_node.cpu_usage == 50.5
        assert worker_node.active_users == 100
        # 验证状态是否正确更新（注意：RunnerState可能使用小写）
        assert worker_node.status.value == "running"
        assert worker_node.last_update > 0.0

    @allure.story("更新状态")
    @allure.title("测试处理无效状态字符串")
    @allure.severity(allure.severity_level.NORMAL)
    def test_update_from_heartbeat_invalid_status(self):
        """测试处理无效状态字符串"""
        node_id = "test_worker"
        worker_node = WorkerNode(node_id)
        original_status = worker_node.status

        # 心跳数据（包含无效状态）
        heartbeat_data = {
            "cpu_percent": 50.5,
            "active_users": 100,
            "status": "invalid_status"
        }

        # 更新状态
        worker_node.update_from_heartbeat(heartbeat_data)

        assert worker_node.cpu_usage == 50.5
        assert worker_node.active_users == 100
        # 验证状态保持不变
        assert worker_node.status == original_status
        assert worker_node.last_update > 0.0

    @allure.story("状态检查")
    @allure.title("测试节点是否过期")
    @allure.severity(allure.severity_level.NORMAL)
    def test_is_stale(self):
        """测试节点是否处于过期状态"""
        node_id = "test_worker"
        worker_node = WorkerNode(node_id)

        # 从未收到心跳，应该返回True
        assert worker_node.is_stale(1.0) is True

        # 更新心跳数据
        heartbeat_data = {
            "cpu_percent": 50.5,
            "active_users": 100
        }
        worker_node.update_from_heartbeat(heartbeat_data)

        # 刚更新，不应该过期
        assert worker_node.is_stale(1.0) is False


@allure.feature("LocalRunner")
class TestLocalRunner:
    """LocalRunner 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试LocalRunner初始化")
    @allure.severity(allure.severity_level.CRITICAL)
    def test_local_runner_initialization(self, mock_config, test_load_shape):
        """测试LocalRunner初始化时正确设置属性"""
        # 使用真实参数进行测试
        user_types = []

        # 使用fixture创建配置
        config = mock_config(prometheus_port=8000)

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, test_load_shape, config)

        assert runner.node == NODE_TYPE_LOCAL
        assert runner.metrics_collector is None
        assert runner.prometheus_server_started is False
        assert runner.prometheus_runner is None

    @allure.story("初始化")
    @allure.title("测试LocalRunner初始化组件")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_local_runner_initialize(self, mock_config, test_load_shape):
        """测试LocalRunner初始化特有的组件"""
        # 使用真实参数进行测试
        user_types = []

        # 使用fixture创建配置
        config = mock_config(prometheus_port=8002)  # 使用不同端口避免冲突

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, test_load_shape, config)

        # 初始化
        await runner.initialize()

        # 验证初始化
        assert runner.metrics_collector is not None
        assert runner.prometheus_runner is not None
        assert runner.prometheus_server_started is True

        # 清理资源
        await runner.stop()

    @allure.story("停止")
    @allure.title("测试LocalRunner停止")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_local_runner_stop(self, mock_config, test_load_shape):
        """测试LocalRunner停止"""
        # 使用真实参数进行测试
        user_types = []

        # 使用fixture创建配置
        config = mock_config(prometheus_port=8001)  # 使用不同端口避免冲突

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, test_load_shape, config)

        # 初始化
        await runner.initialize()

        # 手动设置prometheus_server_started为True（确保它被正确设置）
        runner.prometheus_server_started = True

        # 直接测试停止Prometheus服务的逻辑
        if runner.prometheus_runner:
            await runner.prometheus_runner.cleanup()
            runner.prometheus_runner = None
            runner.prometheus_server_started = False

        # 验证停止后状态
        assert runner.prometheus_server_started is False
        assert runner.prometheus_runner is None

    @allure.story("停止")
    @allure.title("测试LocalRunner从READY状态停止")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_local_runner_stop_from_ready(self):
        """测试LocalRunner从READY状态停止"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8002  # 使用不同端口避免冲突
                self.metrics_collection_interval = 5.0

        config = MockConfig()

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, load_shape, config)

        # 直接停止（未初始化）
        await runner.stop()

        # 验证状态未变化
        assert runner.prometheus_server_started is False
        assert runner.prometheus_runner is None

    @allure.story("暂停/恢复")
    @allure.title("测试LocalRunner暂停")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_local_runner_pause(self, mock_config, test_load_shape):
        """测试LocalRunner暂停"""
        # 使用真实参数进行测试
        user_types = []

        # 使用fixture创建配置
        config = mock_config(prometheus_port=8010)  # 使用不同端口避免冲突

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, test_load_shape, config)

        # 初始化
        await runner.initialize()

        # 启动测试
        await runner.start()

        # 模拟测试运行中状态（按照正确的状态转换顺序）
        await runner.state_manager.transition_state(RunnerState.STARTING)
        await runner.state_manager.transition_state(RunnerState.RUNNING)

        # 暂停测试
        await runner.pause()

        # 验证状态
        assert runner.state.value == "paused"

        # 清理资源
        await runner.quit()

    @allure.story("暂停/恢复")
    @allure.title("测试LocalRunner恢复")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_local_runner_resume(self, mock_config, test_load_shape):
        """测试LocalRunner恢复"""
        # 使用真实参数进行测试
        user_types = []

        # 使用fixture创建配置
        config = mock_config(prometheus_port=8011)  # 使用不同端口避免冲突

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, test_load_shape, config)

        # 初始化
        await runner.initialize()

        # 启动测试
        await runner.start()

        # 模拟测试运行中状态（按照正确的状态转换顺序）
        await runner.state_manager.transition_state(RunnerState.STARTING)
        await runner.state_manager.transition_state(RunnerState.RUNNING)

        # 暂停测试
        await runner.pause()
        assert runner.state.value == "paused"

        # 恢复测试
        await runner.resume()

        # 验证状态
        assert runner.state.value == "running"

        # 清理资源
        await runner.quit()

    @allure.story("暂停/恢复")
    @allure.title("测试LocalRunner从错误状态暂停")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_local_runner_pause_from_invalid_state(self, mock_config, test_load_shape):
        """测试LocalRunner从无效状态暂停"""
        # 使用真实参数进行测试
        user_types = []

        # 使用fixture创建配置
        config = mock_config(prometheus_port=8012)  # 使用不同端口避免冲突

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, test_load_shape, config)

        # 初始化
        await runner.initialize()

        # 直接暂停（未启动）
        await runner.pause()

        # 验证状态未变化
        assert runner.state.value == "ready"

        # 清理资源
        await runner.quit()

    @allure.story("暂停/恢复")
    @allure.title("测试LocalRunner从错误状态恢复")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_local_runner_resume_from_invalid_state(self, mock_config, test_load_shape):
        """测试LocalRunner从无效状态恢复"""
        # 使用真实参数进行测试
        user_types = []

        # 使用fixture创建配置
        config = mock_config(prometheus_port=8013)  # 使用不同端口避免冲突

        # 创建LocalRunner实例
        runner = LocalRunner(user_types, test_load_shape, config)

        # 初始化
        await runner.initialize()

        # 直接恢复（未暂停）
        await runner.resume()

        # 验证状态未变化
        assert runner.state.value == "ready"

        # 清理资源
        await runner.quit()


@allure.feature("WorkerRunner")
class TestWorkerRunner:
    """WorkerRunner 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试WorkerRunner初始化")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_worker_runner_initialization(self):
        """测试WorkerRunner初始化时正确设置属性"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            pass

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.metrics_collection_interval = 5.0
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建WorkerRunner实例
        runner = WorkerRunner(user_types, load_shape, config, redis_client)

        assert runner.node == NODE_TYPE_WORKER
        assert runner.redis_client == redis_client
        assert runner.client_id is not None
        assert runner.coordinator is not None

        # 初始化
        await runner.initialize()

        assert runner.metrics_collection_interval == 5.0
        assert runner.metrics_batch_size == 100
        assert runner.metrics_flush_interval == 1.0
        assert runner.metrics_buffer_size == 10000

        # 清理资源
        await runner.quit()

    @allure.story("负载形状管理")
    @allure.title("测试WorkerRunner负载形状管理器")
    @allure.severity(allure.severity_level.NORMAL)
    def test_worker_runner_load_shape_manager(self):
        """测试WorkerRunner的load_shape_manager属性"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            pass

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.metrics_collection_interval = 5.0
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建WorkerRunner实例
        runner = WorkerRunner(user_types, load_shape, config, redis_client)

        # 验证load_shape_manager返回None
        assert runner.load_shape_manager is None

    @allure.story("初始化")
    @allure.title("测试WorkerRunner初始化组件")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_worker_runner_initialize(self):
        """测试WorkerRunner初始化"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.metrics_collection_interval = 5.0
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建WorkerRunner实例
        runner = WorkerRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 验证初始化
        assert runner.metrics_collector is not None
        assert runner.coordinator is not None

        # 清理资源
        await runner.quit()

    @allure.story("命令处理")
    @allure.title("测试WorkerRunner处理命令")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_worker_runner_handle_command(self):
        """测试WorkerRunner处理命令"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.metrics_collection_interval = 5.0
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建WorkerRunner实例
        runner = WorkerRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 测试startup命令 - 不实际执行apply_load，只测试命令处理逻辑
        # 由于apply_load需要正整数用户数，我们直接测试命令处理函数的其他部分
        # 这里我们不调用_startup_completed，因为它需要Redis连接
        await runner._handle_command({"user_count": 1, "rate": 1}, "worker1", "startup")

        # 测试stop命令
        await runner._handle_command({}, "worker1", "stop")

        # 测试pause命令
        await runner._handle_command({}, "worker1", "pause")

        # 测试resume命令
        await runner._handle_command({}, "worker1", "resume")

        # 测试quit命令
        await runner._handle_command({}, "worker1", "quit")

        # 清理资源
        await runner.quit()

    @allure.story("应用负载")
    @allure.title("测试WorkerRunner应用负载")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_worker_runner_apply_load(self):
        """测试WorkerRunner应用负载"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个更完整的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

            async def hset(self, *args, **kwargs):
                pass

            async def pubsub(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None

                    async def subscribe(self, *args, **kwargs):
                        pass
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.metrics_collection_interval = 5.0
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建WorkerRunner实例
        runner = WorkerRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 清理资源 - 不调用apply_load，因为会创建真实用户
        await runner.quit()


@allure.feature("MasterRunner")
class TestMasterRunner:
    """MasterRunner 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试MasterRunner初始化")
    @allure.severity(allure.severity_level.CRITICAL)
    def test_master_runner_initialization(self):
        """测试MasterRunner初始化时正确设置属性"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            pass

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        assert runner.user_types == user_types
        assert runner.load_shape == load_shape
        assert runner.config == config
        assert runner.node == NODE_TYPE_MASTER
        assert runner.redis_client == redis_client
        assert runner.coordinator is not None
        assert runner.workers == {}
        assert runner.metrics_collector is None
        assert runner.cpu_usage == 0
        assert runner.background_tasks == []
        assert runner._load_shape_manager is None
        assert runner.prometheus_runner is None
        assert runner.prometheus_server_started is False

    @allure.story("初始化")
    @allure.title("测试MasterRunner初始化组件")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_master_runner_initialize(self):
        """测试MasterRunner初始化"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8003  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 验证初始化
        assert runner.metrics_collector is not None
        assert runner.prometheus_runner is not None
        assert runner.prometheus_server_started is True
        assert len(runner.background_tasks) == 3

        # 清理资源
        await runner.quit()

    @allure.story("资源分配")
    @allure.title("测试MasterRunner分配资源")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_master_runner_distribute_resources(self):
        """测试MasterRunner将总资源均匀分配给所有Worker节点"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            pass

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 测试分配资源
        distribution = runner._distribute_resources(10, 3)
        assert len(distribution) == 3
        assert sum(distribution) == 10
        assert 4 in distribution  # 10 / 3 = 3 余 1，所以有一个节点会分配到 4

    @allure.story("命令处理")
    @allure.title("测试MasterRunner处理命令")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_master_runner_handle_command(self):
        """测试MasterRunner处理命令"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8004  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 测试startup_completed命令
        runner._startup_completion_tracker = {
            "completed_workers": set(),
            "expected_workers": 2
        }

        await runner._handle_command({"user_count": 100}, "worker1", "startup_completed")
        assert "worker1" in runner._startup_completion_tracker["completed_workers"]

        # 测试stop命令
        runner._stop_completion_tracker = {
            "stopped_workers": set(),
            "total_workers": 1  # 设置为1，这样处理stop命令时会触发状态转换
        }

        await runner._handle_command({}, "worker1", "stop")
        assert "worker1" in runner._stop_completion_tracker["stopped_workers"]

        # 清理资源
        await runner.quit()

    @allure.story("Worker管理")
    @allure.title("测试MasterRunner更新Worker状态")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_master_runner_update_worker_status(self):
        """测试MasterRunner更新Worker状态和指标数据"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8005  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 测试更新Worker状态
        heartbeat_data = {
            "cpu_percent": 50.5,
            "active_users": 100,
            "status": "running"
        }

        await runner._update_worker_status(heartbeat_data, "worker1")
        assert "worker1" in runner.workers
        assert runner.workers["worker1"].cpu_usage == 50.5
        assert runner.workers["worker1"].active_users == 100
        # 验证状态是否正确更新（注意：RunnerState可能使用小写）
        assert runner.workers["worker1"].status.value == "running"

        # 清理资源
        await runner.quit()

    @allure.story("Worker管理")
    @allure.title("测试MasterRunner获取健康Worker")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_master_runner_get_healthy_workers(self):
        """测试MasterRunner获取所有健康的Worker节点"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

            async def get(self, *args, **kwargs):
                # 模拟Redis get操作，返回一个有效的心跳值
                import json
                return json.dumps(
                    {"timestamp": asyncio.get_event_loop().time()})

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8006  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 添加测试Worker
        runner.workers["worker1"] = WorkerNode("worker1")
        runner.workers["worker1"].status = RunnerState.RUNNING

        # 测试获取健康Worker
        healthy_workers = await runner.get_healthy_workers()
        # 由于我们的Redis模拟可能不会正确实现check_worker_heartbeat，所以这里可能返回空列表
        # 但我们可以验证方法执行没有错误
        assert isinstance(healthy_workers, list)

        # 清理资源
        await runner.quit()

    @allure.story("退出")
    @allure.title("测试MasterRunner退出")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_master_runner_quit(self):
        """测试MasterRunner退出，确保正确停止load_shape_manager"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8009  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 启动负载形状管理器
        await runner.start()
        assert runner.load_shape_manager is not None
        assert runner.load_shape_manager._is_running is True

        # 退出
        await runner.quit()

        # 验证load_shape_manager已停止
        assert runner.load_shape_manager._is_running is False
        assert runner.load_shape_manager._task is None
        assert runner.state_manager.is_in_quit_state()

    @allure.story("暂停/恢复")
    @allure.title("测试MasterRunner暂停")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_master_runner_pause(self):
        """测试MasterRunner暂停"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        published_commands = []

        import json

        class MockRedisClient:
            async def publish(self, channel, message):
                # 解析消息，提取command字段
                try:
                    payload = json.loads(message)
                    if "command" in payload:
                        published_commands.append(payload["command"])
                except json.JSONDecodeError:
                    pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8014  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 启动测试
        await runner.start()

        # 模拟测试运行中状态（按照正确的状态转换顺序）
        await runner.state_manager.transition_state(RunnerState.STARTING)
        await runner.state_manager.transition_state(RunnerState.RUNNING)

        # 暂停测试
        await runner.pause()

        # 验证状态
        assert runner.state_manager.get_current_state().value == "paused"
        # 验证是否发送了pause命令
        assert "pause" in published_commands

        # 清理资源
        await runner.quit()

    @allure.story("暂停/恢复")
    @allure.title("测试MasterRunner恢复")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_master_runner_resume(self):
        """测试MasterRunner恢复"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个简单的Redis客户端模拟
        published_commands = []

        import json

        class MockRedisClient:
            async def publish(self, channel, message):
                # 解析消息，提取command字段
                try:
                    payload = json.loads(message)
                    if "command" in payload:
                        published_commands.append(payload["command"])
                except json.JSONDecodeError:
                    pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8015  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 启动测试
        await runner.start()

        # 模拟测试运行中状态（按照正确的状态转换顺序）
        await runner.state_manager.transition_state(RunnerState.STARTING)
        await runner.state_manager.transition_state(RunnerState.RUNNING)

        # 暂停测试
        await runner.pause()
        assert runner.state_manager.get_current_state().value == "paused"

        # 恢复测试
        await runner.resume()

        # 验证状态
        assert runner.state_manager.get_current_state().value == "running"
        # 验证是否发送了resume命令
        assert "resume" in published_commands

        # 清理资源
        await runner.quit()

    @allure.story("资源分配")
    @allure.title("测试MasterRunner广播启动命令")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_master_runner_broadcast_startup(self):
        """测试MasterRunner广播启动命令到所有Worker节点 - 验证异步方法正确调用"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 记录发布的命令
        published_commands = []

        # 创建一个模拟的Redis客户端
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8007  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 添加测试Worker
        runner.workers["worker1"] = WorkerNode("worker1")
        runner.workers["worker1"].status = RunnerState.READY
        runner.workers["worker2"] = WorkerNode("worker2")
        runner.workers["worker2"].status = RunnerState.READY

        # 模拟coordinator的publish方法，记录发布的命令
        async def mock_publish(channel, data, worker_id=None, command=None):
            published_commands.append({
                "channel": channel,
                "data": data,
                "worker_id": worker_id,
                "command": command
            })

        runner.coordinator.publish = mock_publish

        # 测试广播启动命令 - 10个用户，速率10/s
        await runner._broadcast_startup(10, 10)

        # 验证命令已发布给所有Worker
        assert len(published_commands) == 2

        # 验证资源分配正确（10个用户分配给2个Worker，应该是5和5）
        user_counts = [cmd["data"]["user_count"] for cmd in published_commands]
        assert sum(user_counts) == 10
        assert all(count > 0 for count in user_counts)

        # 验证速率分配正确
        rates = [cmd["data"]["rate"] for cmd in published_commands]
        assert sum(rates) == 10

        # 验证命令类型正确
        assert all(cmd["command"] == "startup" for cmd in published_commands)
        assert all(cmd["channel"] == "command" for cmd in published_commands)

        # 清理资源
        await runner.quit()

    @allure.story("资源分配")
    @allure.title("测试MasterRunner广播启动命令无Worker时抛出异常")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_master_runner_broadcast_startup_no_workers(self):
        """测试MasterRunner广播启动命令时没有Worker时抛出异常"""
        # 使用真实参数进行测试
        user_types = []

        # 创建一个简单的负载形状类
        class TestLoadShape:
            def tick(self):
                return None

        load_shape = TestLoadShape()

        # 创建一个模拟的Redis客户端
        class MockRedisClient:
            async def publish(self, *args, **kwargs):
                pass

            async def subscribe(self, *args, **kwargs):
                class MockPubSub:
                    async def get_message(self, *args, **kwargs):
                        return None
                return MockPubSub()

        redis_client = MockRedisClient()

        class MockConfig:
            def __init__(self):
                self.prometheus_port = 8008  # 使用不同端口避免冲突
                self.metrics_batch_size = 100
                self.metrics_flush_interval = 1.0
                self.metrics_buffer_size = 10000

        config = MockConfig()

        # 创建MasterRunner实例
        runner = MasterRunner(user_types, load_shape, config, redis_client)

        # 初始化
        await runner.initialize()

        # 测试没有Worker时应该抛出异常
        with pytest.raises(RunnerError) as exc_info:
            await runner._broadcast_startup(10, 10)

        assert "No ready workers available" in str(exc_info.value)

        # 清理资源
        await runner.quit()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])


# 测试create_prometheus_app函数
@allure.feature("Prometheus")
class TestPrometheus:
    @allure.story("创建应用")
    @allure.title("测试create_prometheus_app函数")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_create_prometheus_app(self):
        """测试create_prometheus_app函数，验证runner参数功能"""
        # 创建一个模拟的runner对象
        class MockRunner:
            def __init__(self):
                self.state = "running"
                self.metrics_collector = None
                self.worker_status = {}

        # 测试带runner参数的情况
        runner = MockRunner()
        app = create_prometheus_app(runner=runner)
        assert app is not None

        # 测试不带runner参数的情况（向后兼容）
        app_no_runner = create_prometheus_app()
        assert app_no_runner is not None



# encoding: utf-8

import asyncio
import json
from typing import Optional
from uuid import uuid4

from redis.asyncio import Redis
from redis.asyncio.connection import ConnectionPool

from aiotest.logger import logger

# 心跳相关常量
HEARTBEAT_INTERVAL = 1.0  # 心跳监控时间间隔(秒)
HEARTBEAT_LIVENESS = 5    # 心跳存活检查次数

# 批量传输相关常量
MAX_BATCH_SIZE = 1000     # 最大批量大小，防止消息过大


class RedisConnection:
    """
    Redis连接管理器

    功能：
        - 管理Redis连接池
        - 提供单例模式的Redis客户端
        - 处理连接异常和重连
        - 支持连接重试机制和指数退避策略
    """

    def __init__(self, max_retries: int = 3, retry_interval: float = 1.0):
        """
        初始化Redis连接管理器

        参数：
            max_retries: 最大重试次数，默认3次
            retry_interval: 初始重试间隔（秒），默认1秒，后续使用指数退避
        """
        self._pool: Optional[ConnectionPool] = None
        self._client: Optional[Redis] = None
        self.max_retries = max_retries
        self.retry_interval = retry_interval

    async def get_client(self, path: str, port: int, password: str) -> Redis:
        """
        获取 Redis 连接实例。

        参数：
            path (str): Redis 服务器地址。
            port (int): Redis 服务器端口。
            password (str): Redis 服务器密码。

        返回：
            Redis: Redis 客户端实例。

        异常：
            redis.ConnectionError: 如果连接 Redis 失败。
        """
        if self._client is None:
            logger.debug("正在创建新的Redis连接，地址: %s:%s", path, port)
            retry_count = 0
            while retry_count < self.max_retries:
                try:
                    if self._pool is None:
                        logger.debug(
                            "创建新的连接池: %s:%s", path, port)
                        self._pool = ConnectionPool.from_url(
                            f"redis://:{password}@{path}:{port}/0",
                            decode_responses=True
                        )
                    self._client = Redis(connection_pool=self._pool)
                    # 测试连接是否可用
                    logger.debug("正在测试Redis连接，地址: %s:%s", path, port)
                    await self._client.ping()
                    logger.info("Redis连接已建立: %s:%s", path, port)
                    return self._client
                except Exception as e:
                    # 重置连接状态以便重试
                    self._pool = None
                    self._client = None
                    retry_count += 1
                    if retry_count < self.max_retries:
                        # 使用指数退避策略：每次重试间隔翻倍
                        backoff_interval = self.retry_interval * \
                            (2 ** (retry_count - 1))
                        logger.warning(
                            "Redis 连接失败 (尝试 %d/%d): %s。%.2f 秒后重试...",
                            retry_count, self.max_retries, str(e), backoff_interval)
                        await asyncio.sleep(backoff_interval)
                    else:
                        logger.error(
                            "Redis 连接失败，已重试 %d 次: %s",
                            self.max_retries, str(e))
                        raise e
        logger.debug("正在使用现有的Redis连接: %s:%s", path, port)
        return self._client

    async def close(self):
        """关闭Redis连接"""
        if self._client:
            await self._client.close()
        if self._pool:
            await self._pool.disconnect()
        self._client = None
        self._pool = None
        logger.info("Redis连接已关闭")


class DistributedLock:
    """
    Redis 分布式锁。

    功能：
        - 提供基于 Redis 的分布式锁机制。
        - 支持超时和重试机制，防止死锁。

    示例：
        >>> lock = DistributedLock(redis_client, "resource_name", timeout=10)
        >>> async with await lock:
        ...     # 临界区代码
        ...     pass

    参数：
        redis (Redis): Redis 客户端实例。
        lock_key (str): 锁的键名。
        timeout (float): 锁的超时时间（秒）。
        wait_timeout (float): 获取锁的等待超时（秒）。
        retry_interval (float): 获取锁失败后的重试间隔（秒）。
    """

    def __init__(
        self,
        redis: Redis,
        lock_key: str,
        timeout: float = 10.0,
        wait_timeout: float = None,
        retry_interval: float = 0.1
    ):
        """
        初始化分布式锁

        参数:
            redis: Redis 客户端实例
            lock_key: 锁的键名
            timeout: 锁的超时时间(秒)，防止死锁
            wait_timeout: 获取锁的等待超时(秒)，None表示不等待
            retry_interval: 获取锁失败后的重试间隔(秒)
        """
        self.redis = redis
        self.lock_key = f"aiotest:lock:{lock_key}"
        self.timeout = timeout
        self.wait_timeout = wait_timeout
        self.retry_interval = retry_interval
        self.identifier = str(uuid4())
        self.locked = False

    async def acquire(self) -> bool:
        """
        获取分布式锁

        返回:
            bool: 是否成功获取锁
        """
        start_time = asyncio.get_event_loop().time()

        while True:
            # 尝试获取锁 (SET lock_key identifier NX PX timeout)
            acquired = await self.redis.set(
                self.lock_key,
                self.identifier,  # 唯一标识，这是我加的锁
                nx=True,  # 只有当key不存在时才设置
                px=int(self.timeout * 1000)  # 过期时间（毫秒）
            )

            if acquired:
                self.locked = True
                return True

            # 如果不等待或已超时，则返回失败
            if self.wait_timeout is None:
                return False

            elapsed = asyncio.get_event_loop().time() - start_time
            if elapsed >= self.wait_timeout:
                return False

            # 没获取到锁，等待后重试
            await asyncio.sleep(self.retry_interval)

    async def release(self) -> bool:
        """
        释放分布式锁

        返回:
            bool: 是否成功释放锁
        """
        if not self.locked:
            return False

        # 使用Lua脚本保证原子性操作
        lua_script = """
        if redis.call("get", KEYS[1]) == ARGV[1] then
            return redis.call("del", KEYS[1])
        else
            return 0
        end
        """

        released = await self.redis.eval(
            lua_script,
            1,
            self.lock_key,
            self.identifier
        )

        if released:
            self.locked = False
            return True
        return False

    async def __aenter__(self):
        await self.acquire()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        # 参数必须保留，即使不使用（异步上下文管理器协议要求）
        await self.release()

    @staticmethod
    async def with_lock(
        redis: Redis,
        lock_key: str,
        timeout: float = 10.0,
        wait_timeout: float = None
    ):
        """
        锁的快捷使用方式

        示例:
            >>> async with await DistributedLock.with_lock(redis, "my_lock_key") as lock:
            ...     if lock.locked:
            ...         # 临界区代码
            ...         pass
        """
        lock = DistributedLock(redis, lock_key, timeout, wait_timeout)
        await lock.acquire()

        # 返回一个不会重复 acquire 的上下文管理器
        return DistributedLock._LockContext(lock)

    class _LockContext:
        """内部上下文管理器，用于 with_lock 方法"""

        def __init__(self, lock: 'DistributedLock'):
            self.lock = lock

        async def __aenter__(self):
            return self.lock

        async def __aexit__(self, exc_type, exc_val, exc_tb):
            # 参数必须保留，即使不使用（异步上下文管理器协议要求）
            await self.lock.release()


class DistributedCoordinator:
    """
    分布式协调器，基于 Redis 的发布/订阅机制实现跨节点的通信。

    功能：
    1. 统一发布系统：支持命令、指标、心跳数据的发布
    2. 监听系统：监听命令、请求指标和心跳数据
    3. 心跳检查：检查Worker节点存活状态

    使用场景：
    - 分布式任务调度
    - 多节点协同操作
    - 实时事件通知
    - 节点健康监控

    示例：
        >>> coordinator = DistributedCoordinator(redis_client, role="master")
        >>> async def handle_command(data, worker_id, command):
        ...     print(f"Received command: {command}, data: {data}, from: {worker_id}")
        >>> task = asyncio.create_task(coordinator.listen_commands(handle_command))
        >>>
        >>> # 发布命令
        >>> await coordinator.publish("command", {"task_id": 123}, command="start_task")
        >>> await coordinator.publish("command", {"reason": "maintenance"}, worker_id="worker_1", command="stop")
        >>>
        >>> # 发送心跳
        >>> await coordinator.publish("heartbeat", {"cpu_percent": 45.0, "active_users": 100})
        >>>
        >>> # 发布请求指标
        >>> await coordinator.publish("request_metrics", request_data, worker_id="worker_1")
        >>>
        >>> # Master监听
        >>> heartbeat_task = asyncio.create_task(coordinator.listen_heartbeats(handle_heartbeat))
    """

    def __init__(self, redis: Redis, role: str = "master",
                 node_id: str = None):
        """
        初始化分布式协调器。

        参数：
            redis: Redis 客户端实例，用于发布/订阅消息。
            role: 节点角色，可选值为 "master" 或 "worker"。
            node_id: 节点ID，可选。
        """
        self.redis = redis
        self.role = role
        self.node_id = node_id or f"{role}_{id(self)}"
        self.pubsub = None
        self.handler = None

        # 根据角色确定订阅和发布的频道
        if role == "master":
            self.subscribe_channel = "aiotest:command:worker_to_master"  # 订阅Worker发来的消息
            self.publish_channel = "aiotest:command:master_to_worker"  # 发布给Worker的消息
        elif role == "worker":
            self.subscribe_channel = "aiotest:command:master_to_worker"  # 订阅Master发来的消息
            self.publish_channel = "aiotest:command:worker_to_master"  # 发布给Master的消息
        else:
            raise ValueError("无效的角色。必须是 'master' 或 'worker'。")

    async def publish(self, channel_type: str, data: dict,
                      worker_id: str = None, **kwargs):
        """
        统一的数据发布方法，支持多种数据类型的发布

        参数：
            channel_type: 频道类型，支持 "command", "request_metrics", "heartbeat"
            data: 要发布的数据字典
            worker_id: Worker节点ID
                       - command: 可选，用于指定目标worker
                       - request_metrics: 必需
                       - heartbeat: 不使用（使用self.node_id）
            **kwargs: 额外参数，如command参数用于命令发布

        返回：
            None

        使用示例：
            # 发布命令
            await coordinator.publish("command", {"reason": "maintenance"}, worker_id="worker_1", command="stop")

            # 发布批量请求指标
            await coordinator.publish("request_metrics", batch_data, worker_id="worker_1")

            # 发送心跳
            await coordinator.publish("heartbeat", {"cpu_percent": 45.0, "active_users": 100})
        """
        if channel_type == "command":
            # 发布命令到命令频道
            command = kwargs.get("command")
            if not command:
                raise ValueError(
                    "命令发布需要 'command' 参数")

            payload = {"command": command}
            if data:
                payload["data"] = data
            if worker_id:
                payload["worker_id"] = worker_id
            await self.redis.publish(self.publish_channel, json.dumps(payload))

        elif channel_type == "request_metrics":
            # 发布批量请求数据到请求指标频道
            if not worker_id:
                raise ValueError(
                    "请求指标批量发布需要 'worker_id'")

            # 如果批量数据过大，拆分成多个小批次发送
            if isinstance(data, list) and len(data) > MAX_BATCH_SIZE:
                for i in range(0, len(data), MAX_BATCH_SIZE):
                    batch_chunk = data[i:i + MAX_BATCH_SIZE]
                    batch_dict = {
                        'batch': batch_chunk,
                        'worker_id': worker_id,
                        'timestamp': asyncio.get_event_loop().time(),
                        'chunk_index': i // MAX_BATCH_SIZE,
                        'total_chunks': (len(data) + MAX_BATCH_SIZE - 1) // MAX_BATCH_SIZE
                    }
                    await self.redis.publish("aiotest:metrics", json.dumps(batch_dict))
            else:
                batch_dict = {
                    'batch': data,
                    'worker_id': worker_id,
                    'timestamp': asyncio.get_event_loop().time()
                }
                await self.redis.publish("aiotest:metrics", json.dumps(batch_dict))

        elif channel_type == "heartbeat":
            # 发送心跳数据（存储到Redis hash，不是发布）
            heartbeat_data = {
                **data,
                "timestamp": int(data.get('timestamp', asyncio.get_event_loop().time()))
            }

            # 将心跳数据存储到Redis，设置过期时间
            heartbeat_key = f"aiotest:heartbeat:{self.node_id}"
            await self.redis.hset(heartbeat_key, mapping=heartbeat_data)
            await self.redis.expire(heartbeat_key, int(HEARTBEAT_INTERVAL * HEARTBEAT_LIVENESS * 2))

        else:
            raise ValueError("不支持的 channel_type: %s", channel_type)

    async def check_worker_heartbeat(self, worker_id: str) -> bool:
        """
        检查特定Worker的心跳状态（Master端使用）

        参数：
            worker_id: Worker节点ID

        返回：
            bool: Worker是否存活
        """
        heartbeat_key = f"aiotest:heartbeat:{worker_id}"
        heartbeat_data = await self.redis.hgetall(heartbeat_key)

        if not heartbeat_data:
            return False

        try:
            current_time = asyncio.get_event_loop().time()
            last_heartbeat = float(heartbeat_data.get("timestamp", 0))
            return (current_time - last_heartbeat) <= HEARTBEAT_INTERVAL * \
                HEARTBEAT_LIVENESS
        except (ValueError, TypeError):
            return False

    async def listen_heartbeats(self, callback=None):
        """
        监听Worker心跳数据变化（Master端使用）

        参数：
            callback: 可选的回调函数，格式为 async def callback(heartbeat_data: dict, worker_id: str)
        """
        # 注意：Redis的心跳数据是存储在hash中的，不是通过pub/sub发布的
        # 所以这里提供轮询检查心跳变化的机制
        worker_status = {}  # 记录上一次的worker状态

        try:
            while True:
                try:
                    heartbeat_keys = []
                    cursor = 0
                    while True:
                        cursor, keys = await self.redis.scan(cursor, match="aiotest:heartbeat:*", count=100)
                        heartbeat_keys.extend(keys)
                        if cursor == 0:
                            break

                    for key in heartbeat_keys:
                        worker_id = key.replace("aiotest:heartbeat:", "")
                        heartbeat_data = await self.redis.hgetall(key)

                        if heartbeat_data:
                            # 检查状态是否发生变化
                            if worker_id not in worker_status or worker_status[worker_id] != heartbeat_data:
                                worker_status[worker_id] = heartbeat_data

                                # 调用回调函数（如果提供）
                                if callback:
                                    await callback(heartbeat_data, worker_id)

                    await asyncio.sleep(HEARTBEAT_INTERVAL)

                except asyncio.CancelledError:
                    # 任务被取消，记录日志并退出
                    logger.info("心跳监听器任务已取消")
                    raise
                except Exception as e:
                    logger.error("心跳监听器错误: %s", str(e))
                    await asyncio.sleep(HEARTBEAT_INTERVAL)
        except asyncio.CancelledError:
            # 任务被取消，记录日志并退出
            logger.info("心跳监听器已停止")
            raise

    async def listen_request_metrics(self, callback=None):
        """
        监听Worker上报的请求数据

        参数：
            callback: 可选的回调函数，格式为 async def callback(metrics_data: dict, worker_id: str)
                      如果不提供，数据将只通过事件系统处理
        """
        request_pubsub = None
        try:
            request_pubsub = self.redis.pubsub()

            # 订阅请求数据channel
            await request_pubsub.subscribe("aiotest:metrics")
            logger.info(
                "%s 开始从 Redis 监听请求指标", self.role)

            while True:  # 持续监听，由外部控制停止
                try:
                    # 获取请求数据消息
                    request_message = await request_pubsub.get_message(timeout=1.0)

                    # 处理请求数据
                    if request_message and request_message['type'] == 'message':
                        try:
                            message_data = json.loads(request_message['data'])

                            # 处理批量数据
                            batch = message_data.get('batch', [])
                            worker_id = message_data.get(
                                'worker_id', 'unknown')
                            chunk_index = message_data.get('chunk_index', 0)
                            total_chunks = message_data.get('total_chunks', 1)

                            # 记录分块信息（如果是分块传输）
                            if total_chunks > 1:
                                logger.debug(
                                    "处理指标分块 %d/%d，来自 %s",
                                    chunk_index + 1, total_chunks, worker_id)

                            # 遍历批量数据并调用回调
                            for metrics_data in batch:
                                if callback:
                                    await callback(metrics_data, worker_id)

                        except (json.JSONDecodeError, KeyError, Exception) as e:
                            if isinstance(e, (json.JSONDecodeError, KeyError)):
                                logger.warning(
                                    "从 Redis 解析请求指标失败: %s",
                                    str(e))
                            else:
                                logger.warning(
                                    "处理请求指标回调失败: %s",
                                    str(e))

                except asyncio.CancelledError:
                    # 任务被取消，记录日志并退出
                    logger.info("请求指标监听器任务已取消")
                    raise
                except Exception as e:
                    logger.warning("Redis消息处理错误: %s", str(e))
                    await asyncio.sleep(0.1)

        except asyncio.CancelledError:
            # 任务被取消，记录日志并退出
            logger.info("请求指标监听器已停止")
            raise
        except Exception as e:
            logger.error("Redis请求指标监听器失败: %s", str(e))
        finally:
            # 清理pubsub连接
            if request_pubsub:
                try:
                    await request_pubsub.unsubscribe("aiotest:metrics")
                    await request_pubsub.close()
                except Exception as e:
                    logger.warning(
                        "清理请求指标 pubsub 时出错: %s",
                        str(e))
            logger.info("Redis请求指标监听器已停止")

    async def listen_commands(self, command_handler):
        """
        统一的命令监听器

        参数：
            command_handler: 命令处理函数，格式为 async def handler(data: dict, worker_id: str, command: str)
        """
        pubsub = None
        try:
            pubsub = self.redis.pubsub()
            await pubsub.subscribe(self.subscribe_channel)
            logger.info(
                "%s 开始在 %s 上监听命令", self.role, self.subscribe_channel)

            # 持续监听，直到收到quit命令或任务被取消
            while True:
                try:
                    # 获取消息，使用async for循环来监听，这样更可靠
                    async for message in pubsub.listen():
                        if message["type"] == "message":
                            try:
                                data = json.loads(message["data"])
                                command = data.get("command")
                                command_data = data.get("data")
                                worker_id = data.get("worker_id")

                                logger.info(
                                    "收到命令: %s, 数据: %s, worker_id: %s",
                                    command, command_data, worker_id)

                                # 只处理发给当前节点的命令（如果有worker_id过滤）
                                # Master节点接收所有Worker节点的命令，Worker节点只接收发给自己的命令
                                if self.role == "worker" and worker_id and hasattr(
                                        self, 'node_id') and worker_id != self.node_id:
                                    logger.info(
                                        "命令不是发给当前worker，跳过。worker_id: %s, self.node_id: %s",
                                        worker_id, self.node_id)
                                    continue

                                # 调用统一的命令处理器
                                logger.info("正在处理命令: %s", command)
                                await command_handler(command_data, worker_id, command)

                                # 如果是quit命令，退出循环
                                if command == "quit":
                                    logger.info(
                                        "收到退出命令，退出命令监听器")
                                    break

                            except json.JSONDecodeError:
                                logger.warning(
                                    "解析命令消息失败: %s", message)
                                continue
                            except Exception as e:
                                logger.error(
                                    "处理命令消息错误: %s", e)
                                continue
                        elif message["type"] == "subscribe":
                            # Redis 服务器确认订阅成功
                            logger.info(
                                "已订阅频道: %s",
                                message['channel'])
                        elif message["type"] == "unsubscribe":
                            # Redis 服务器确认取消订阅成功
                            logger.info(
                                "已取消订阅频道: %s",
                                message['channel'])
                            break

                except asyncio.CancelledError:
                    # 任务被取消，记录日志
                    logger.info("命令监听器任务已取消")
                    break
                except Exception as e:
                    logger.error("命令监听器错误: %s", str(e))
                    # 出错后，等待一段时间再继续监听
                    await asyncio.sleep(1.0)
                    continue

        except Exception as e:
            logger.error("命令监听器错误: %s", str(e))
        finally:
            if pubsub:
                try:
                    await pubsub.unsubscribe(self.subscribe_channel)
                    await pubsub.close()
                except Exception as e:
                    logger.warning(
                        "清理命令 pubsub 时出错: %s",
                        str(e))
            logger.info("命令监听器已停止")



# encoding: utf-8

import asyncio
import os
import sys

import allure
import pytest

from aiotest.main import main

# 测试文件内容模板
test_file_content_with_users = """
from aiotest.users import User
from aiotest.shape import LoadUserShape

class TestUser(User):
    weight = 1
    async def run(self):
        pass

class TestShape(LoadUserShape):
    def tick(self):
        pass

async def on_init_redis():
    pass

init_events.add_handler(on_init_redis)
"""

test_file_content_no_users = """
from aiotest.shape import LoadUserShape

class TestShape(LoadUserShape):
    def tick(self):
        pass
"""

test_file_content_with_events = """
from aiotest.users import User
from aiotest.shape import LoadUserShape

class TestUser(User):
    weight = 1
    async def run(self):
        pass

class TestShape(LoadUserShape):
    def tick(self):
        return (1, 1)

async def on_init_events():
    pass
"""

test_file_content_with_print = """
from aiotest.users import User
from aiotest.shape import LoadUserShape

class TestUser(User):
    weight = 1
    async def run(self):
        pass

class TestShape(LoadUserShape):
    def tick(self):
        return (1, 1)

async def on_init_events():
    print("Init events called")

init_events.add_handler(on_init_events)
"""

test_file_content_no_jobs = """
from aiotest.users import User
from aiotest.shape import LoadUserShape

class TestUser(User):
    weight = 1
    # 没有以 test_ 开头或 _test 结尾的任务函数

class TestShape(LoadUserShape):
    def tick(self):
        return (1, 1)
"""

test_file_content_with_valid_jobs = """
from aiotest.users import User
from aiotest.shape import LoadUserShape

class TestUser(User):
    weight = 1

    async def test_get_request(self):
        pass

    async def test_post_request(self):
        pass

class TestShape(LoadUserShape):
    def tick(self):
        return (1, 1)
"""


@allure.feature("主函数测试")
class TestMain:
    """测试main函数"""

    @allure.story("命令行参数")
    @allure.title("测试显示用户权重选项")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_show_users_weight(self):
        """测试显示用户权重选项"""
        # 创建临时测试文件
        test_file = "test_main_show_weight.py"
        try:
            with open(test_file, 'w') as f:
                f.write(test_file_content_with_users)
            # 测试显示用户权重选项
            sys.argv = ["aiotest", "-f", test_file, "--show-users-wight"]
            with pytest.raises(SystemExit):
                asyncio.run(main())
        finally:
            # 清理临时文件
            if os.path.exists(test_file):
                os.remove(test_file)

    @allure.story("命令行参数")
    @allure.title("测试无用户类的情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_no_user_classes(self):
        """测试无用户类的情况"""
        # 创建临时测试文件，只包含形状类
        test_file = "test_main_no_user.py"
        try:
            with open(test_file, 'w') as f:
                f.write(test_file_content_no_users)
            # 测试无用户类的情况
            sys.argv = ["aiotest", "-f", test_file]
            with pytest.raises(SystemExit):
                asyncio.run(main())
        finally:
            # 清理临时文件
            if os.path.exists(test_file):
                os.remove(test_file)

    @allure.story("异常处理")
    @allure.title("测试文件不存在的情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_file_not_found(self):
        """测试文件不存在的情况"""
        # 测试文件不存在的情况
        sys.argv = ["aiotest", "-f", "non_existent_file.py"]
        with pytest.raises(SystemExit):
            asyncio.run(main())

    @allure.story("异常处理")
    @allure.title("测试非Python文件的情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_non_python_file(self):
        """测试非Python文件的情况"""
        # 创建临时非Python文件
        test_file = "test_main_non_python.txt"
        try:
            with open(test_file, 'w') as f:
                f.write("test content")
            # 测试非Python文件的情况
            sys.argv = ["aiotest", "-f", test_file]
            with pytest.raises(SystemExit):
                asyncio.run(main())
        finally:
            # 清理临时文件
            if os.path.exists(test_file):
                os.remove(test_file)

    @allure.story("文件描述符限制")
    @allure.title("测试文件描述符限制检查")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_file_descriptor_limit(self):
        """测试文件描述符限制检查"""
        # 创建临时测试文件
        test_file = "test_main_fd_limit.py"
        try:
            with open(test_file, 'w') as f:
                f.write(test_file_content_with_events)
            # 测试文件描述符限制检查（通过模拟非Windows环境）
            import os
            original_os_name = os.name
            try:
                # 模拟非Windows环境
                os.name = "posix"
                # 测试文件描述符限制检查
                sys.argv = ["aiotest", "-f", test_file, "--show-users-wight"]
                with pytest.raises(SystemExit):
                    asyncio.run(main())
            finally:
                # 恢复原始环境
                os.name = original_os_name
        finally:
            # 清理临时文件
            if os.path.exists(test_file):
                os.remove(test_file)

    @allure.story("初始化事件")
    @allure.title("测试初始化事件触发")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_init_events(self):
        """测试初始化事件触发"""
        # 创建临时测试文件
        test_file = "test_main_init_events.py"
        try:
            with open(test_file, 'w') as f:
                f.write(test_file_content_with_print)
            # 测试初始化事件触发
            sys.argv = ["aiotest", "-f", test_file, "--show-users-wight"]
            with pytest.raises(SystemExit):
                asyncio.run(main())
        finally:
            # 清理临时文件
            if os.path.exists(test_file):
                os.remove(test_file)

    @allure.story("任务函数校验")
    @allure.title("测试用户类没有任务函数的情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_no_jobs(self):
        """测试用户类没有任务函数的情况"""
        # 创建临时测试文件，用户类没有以 test_ 开头或 _test 结尾的任务函数
        test_file = "test_main_no_jobs.py"
        try:
            with open(test_file, 'w') as f:
                f.write(test_file_content_no_jobs)
            # 测试用户类没有任务函数的情况
            sys.argv = ["aiotest", "-f", test_file, "--show-users-wight"]
            with pytest.raises(SystemExit):
                asyncio.run(main())
        finally:
            # 清理临时文件
            if os.path.exists(test_file):
                os.remove(test_file)

    @allure.story("任务函数校验")
    @allure.title("测试用户类有有效任务函数的情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_main_with_valid_jobs(self):
        """测试用户类有有效任务函数的情况"""
        # 创建临时测试文件，用户类有以 test_ 开头的任务函数
        test_file = "test_main_with_valid_jobs.py"
        try:
            with open(test_file, 'w') as f:
                f.write(test_file_content_with_valid_jobs)
            # 测试用户类有有效任务函数的情况
            sys.argv = ["aiotest", "-f", test_file, "--show-users-wight"]
            with pytest.raises(SystemExit):
                asyncio.run(main())
        finally:
            # 清理临时文件
            if os.path.exists(test_file):
                os.remove(test_file)


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8
"""
AioTest性能指标模块

提供基于Prometheus的性能指标收集和报告功能，支持HTTP请求监控、
节点资源监控和自定义业务指标。

特性：
- 事件驱动的指标收集
- 统一的指标管理（请求数据 + 节点指标数据 + 用户活动数据）
- 解耦的架构设计
- 分布式节点支持

使用示例：
    # 初始化指标收集器
    collector = init_unified_collector("local", node_id="worker1")

    # 启动指标收集器（在异步上下文中）
    await collector.start()

    # 触发请求指标事件
    await request_metrics.fire(metrics=request_data)

    # 记录节点指标（Local/Master节点由Runner定期调用，Worker节点通过心跳系统传递）
    metrics_data = {
        "cpu_percent": 45.2,
        "active_users": 10,
        "worker_id": "worker1"
    }
    await collector.record_node_metrics(metrics_data)

    # 停止指标收集器
    await collector.stop()
"""

import asyncio
import time
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from prometheus_client import (
    CollectorRegistry,
    Counter,
    Gauge,
    Histogram,
    generate_latest,
)

from aiotest.events import request_metrics
from aiotest.logger import logger

# 自定义指标注册表，用于隔离指标
REGISTRY = CollectorRegistry()


@dataclass
class RequestMetrics:
    """请求指标数据结构"""
    request_id: str
    method: str
    endpoint: str
    status_code: int = 0
    duration: float = 0.0  # 单位秒
    response_size: int = 0  # 响应体大小（字节）
    error: Optional[Dict[str, Any]] = None
    extra: Optional[Dict[str, Any]] = None
    timestamp: float = field(default_factory=time.time)
    assertion_result: str = "unknown"  # "pass" 或 "fail"


# Prometheus 指标定义
REQUEST_COUNTER = Counter(
    'aiotest_http_requests_total',
    'Total HTTP requests',
    ['method', 'endpoint', 'status_code', 'assertion_result'],
    registry=REGISTRY
)

REQUEST_DURATION = Histogram(
    'aiotest_http_request_duration_seconds',
    'HTTP request duration in seconds',
    ['method', 'endpoint'],
    buckets=[0.1, 0.25, 0.5, 0.8, 1.0, 2.5, 5.0, 10.0, 30.0],
    registry=REGISTRY
)

RESPONSE_SIZE = Histogram(
    'aiotest_http_response_size_bytes',
    'HTTP response size in bytes',
    ['method', 'endpoint'],
    buckets=(100, 1000, 10000, 100000, 1e6),
    registry=REGISTRY
)

WORKER_CPU_USAGE = Gauge(
    'aiotest_worker_cpu_percent',
    'Worker CPU usage percentage',
    ['worker_id', 'machine_id'],
    registry=REGISTRY
)

WORKER_ACTIVE_USERS = Gauge(
    'aiotest_worker_active_users',
    'Number of active users on worker',
    ['worker_id'],
    registry=REGISTRY
)

# 记录断言失败的信息 ：当请求中的断言失败时，会触发 ERROR_COUNTER 记录
# 记录其他类型的错误 ：包括网络错误、超时错误、HTTP 错误 ：4xx、5xx 状态码（当这些被断言为失败时）等
# 提供详细的错误分类 ：通过多个标签提供错误的详细信息
# 查看接口的响应数据 ：错误消息中包含接口的响应数据，便于调试
ERROR_COUNTER = Counter(
    'aiotest_errors_total',
    'Total errors with detailed information',
    ['error_type', 'method', 'endpoint', 'status_code', 'error_message'],
    registry=REGISTRY
)


class MetricsCollector:
    """统一的指标收集器"""

    def __init__(self, node_type: str = "local", redis_client=None, node_id: str = "local",
                 coordinator=None, batch_size: int = 100, flush_interval: float = 1.0,
                 buffer_size: int = 10000):
        """
        初始化指标收集器

        参数：
            node_type: 节点类型 (local/master/worker)
            redis_client: Redis客户端
            node_id: 节点ID
            coordinator: 分布式协调器
            batch_size: 批量上传的大小
            flush_interval: 刷新间隔（秒）
            buffer_size: 本地缓冲区大小
        """
        self.node_type = node_type
        self.node_id = node_id
        self.redis_client = redis_client
        self.coordinator = coordinator

        # 批量上传配置
        self.batch_size = batch_size
        self.flush_interval = flush_interval
        self.buffer_size = buffer_size

        # 本地缓冲区
        self._metrics_buffer: List[Dict[str, Any]] = []
        self._buffer_lock = asyncio.Lock()
        self._flush_task = None

    async def start(self):
        """启动指标收集器"""
        await self._register_event_handlers()

        # 启动定期刷新任务
        if self.node_type == "worker":
            self._flush_task = asyncio.create_task(self._flush_buffer())

        logger.info("指标收集器已启动，节点ID: %s", self.node_id)

    async def stop(self):
        """停止指标收集器"""
        # 停止定期刷新任务
        if self._flush_task:
            self._flush_task.cancel()
            try:
                await self._flush_task
            except asyncio.CancelledError:
                pass

        # 最后一次刷新缓冲区
        if self.node_type == "worker" and self._metrics_buffer:
            await self._do_flush()

        logger.info("指标收集器已停止，节点ID: %s", self.node_id)

    async def _flush_buffer(self):
        """定期刷新缓冲区"""
        while True:
            try:
                await asyncio.sleep(self.flush_interval)
                await self._do_flush()
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.warning("刷新指标缓冲区失败: %s", e)

    async def _do_flush(self):
        """执行缓冲区刷新"""
        async with self._buffer_lock:
            if not self._metrics_buffer:
                return

            # 批量处理缓冲区数据
            # 如果缓冲区数据量 >= batch_size，取 batch_size 条数据
            # 如果缓冲区数据量 < batch_size，取所有数据（避免延迟）
            batch_size = min(len(self._metrics_buffer), self.batch_size)
            batch = self._metrics_buffer[:batch_size]
            self._metrics_buffer = self._metrics_buffer[batch_size:]

        if batch:
            try:
                await self._forward_batch_to_redis(batch)
            except Exception as e:
                logger.warning("转发指标批次失败: %s", e)
                # 失败时将数据放回缓冲区
                async with self._buffer_lock:
                    self._metrics_buffer = batch + self._metrics_buffer

    async def record_node_metrics(self, metrics_data: dict) -> None:
        """
        记录节点指标数据（统一接口，支持分布式和本地模式）

        参数：
            metrics_data: 节点指标数据字典，包含cpu_percent, active_users, status等
                      - 对于Local/Master节点：定期上报的CPU和用户数
                      - 对于Worker节点：心跳系统传递的数据

        说明：
            - Local/Master节点：直接记录自身指标到Prometheus
            - Master节点：还会记录来自Worker节点的心跳数据
            - Worker节点：不直接调用此方法，而是通过心跳系统将数据传递给Master
        """
        try:
            cpu_percent = float(metrics_data.get('cpu_percent', 0.0))
            active_users = int(metrics_data.get('active_users', 0))
            worker_id = metrics_data.get('worker_id', self.node_id)
            machine_id = metrics_data.get('machine_id', 'unknown')

            # 记录到Prometheus
            self._record_cpu_usage(worker_id, machine_id, cpu_percent)
            self._record_active_users(worker_id, active_users)

        except (ValueError, TypeError) as e:
            logger.warning("记录节点指标失败: %s", e)

    def _record_cpu_usage(self, worker_id: str, machine_id: str, cpu_percent: float) -> None:
        """记录CPU使用率指标"""
        WORKER_CPU_USAGE.labels(
            worker_id=worker_id,
            machine_id=machine_id).set(cpu_percent)

    def _record_active_users(self, worker_id: str, active_users: int) -> None:
        """记录活跃用户数指标"""
        WORKER_ACTIVE_USERS.labels(worker_id=worker_id).set(active_users)

    async def _register_event_handlers(self):
        """注册指标事件处理器"""
        await request_metrics.add_handler(self.process_request_metrics)

    async def process_request_metrics(self, **kwargs) -> None:
        """
        处理请求数据（统一接口）

        根据节点类型选择不同的上报方式：
        - local: 直接上报到 Prometheus
        - worker: 转发到 Redis 供 Master 收集

        参数：
            metrics: RequestMetrics 对象
        """
        try:
            # 获取metrics数据
            metrics = kwargs.get('metrics')
            if not metrics:
                return

            # 数据上报（根据节点类型）
            if self.node_type in ("local", "master"):
                # Local 或 Master 节点：直接上报到 Prometheus
                self._report_to_prometheus_from_metrics(metrics)
            elif self.node_type == "worker":
                # Worker 节点：添加到本地缓冲区，等待批量发送到 Redis
                await self._add_metrics_to_buffer(metrics)

        except Exception as e:
            logger.warning("处理请求指标失败: %s", e)

    def _report_to_prometheus_from_metrics(
            self, metrics: RequestMetrics) -> None:
        """从 RequestMetrics 对象上报数据到 Prometheus"""
        method = metrics.method
        endpoint = metrics.endpoint
        status_code = str(metrics.status_code)
        duration = metrics.duration
        response_size = metrics.response_size
        assertion_result = metrics.assertion_result

        # 记录 Prometheus 指标
        self._record_request_counter(method, endpoint, status_code, assertion_result)
        self._record_request_duration(method, endpoint, duration)
        self._record_response_size(method, endpoint, response_size)

        # 记录错误指标
        if metrics.error:
            self._record_error_metrics(metrics, method, endpoint, status_code)

    def _record_request_counter(self, method: str, endpoint: str, status_code: str,
                               assertion_result: str) -> None:
        """记录请求计数指标"""
        REQUEST_COUNTER.labels(
            method=method,
            endpoint=endpoint,
            status_code=status_code,
            assertion_result=assertion_result
        ).inc()

    def _record_request_duration(self, method: str, endpoint: str, duration: float) -> None:
        """记录请求耗时指标"""
        REQUEST_DURATION.labels(
            method=method,
            endpoint=endpoint
        ).observe(duration)

    def _record_response_size(self, method: str, endpoint: str, response_size: int) -> None:
        """记录响应大小指标"""
        RESPONSE_SIZE.labels(
            method=method,
            endpoint=endpoint
        ).observe(response_size)

    def _record_error_metrics(self, metrics: RequestMetrics, method: str, endpoint: str,
                             status_code: str) -> None:
        """记录错误指标"""
        error_message = str(metrics.error.get('message', 'unknown'))

        # 限制错误消息长度，避免过长
        if len(error_message) > 200:
            error_message = error_message[:200]

        ERROR_COUNTER.labels(
            error_type=metrics.error.get('exc_type', 'unknown'),
            method=method,
            endpoint=endpoint,
            status_code=status_code,
            error_message=error_message
        ).inc()

    async def _add_metrics_to_buffer(self, metrics: RequestMetrics) -> None:
        """
        从 RequestMetrics 对象转换数据并添加到本地缓冲区，等待批量发送到 Redis

        参数：
            metrics: RequestMetrics 对象，包含请求相关的指标数据
        """
        if not self.coordinator:
            return

        # 转换为字典格式并添加 worker 信息
        metrics_dict = self._convert_metrics_to_dict(metrics)

        # 添加到缓冲区
        await self._append_to_buffer(metrics_dict)

    def _convert_metrics_to_dict(self, metrics: RequestMetrics) -> Dict[str, Any]:
        """将RequestMetrics对象转换为字典格式"""
        return {
            'request_id': metrics.request_id,
            'method': metrics.method,
            'endpoint': metrics.endpoint,
            'status_code': metrics.status_code,
            'duration': metrics.duration,
            'response_size': metrics.response_size,
            'error': metrics.error,
            'timestamp': metrics.timestamp,
            'assertion_result': metrics.assertion_result,
            'worker_id': self.node_id
        }

    async def _append_to_buffer(self, metrics_dict: Dict[str, Any]) -> None:
        """将指标数据添加到缓冲区"""
        async with self._buffer_lock:
            if len(self._metrics_buffer) < self.buffer_size:
                self._metrics_buffer.append(metrics_dict)
            else:
                # 缓冲区已满，丢弃最旧的数据
                self._metrics_buffer.pop(0)
                self._metrics_buffer.append(metrics_dict)
                logger.warning("指标缓冲区已满，丢弃最旧的指标")

    async def _forward_batch_to_redis(
            self, batch: List[Dict[str, Any]]) -> None:
        """批量转发数据到 Redis"""
        if not self.coordinator:
            return

        # 批量发布请求数据
        await self.coordinator.publish("request_metrics", batch, worker_id=self.node_id)

    def get_metrics_export(self) -> str:
        """获取 Prometheus 格式的指标导出，仅测试使用"""
        try:
            return generate_latest(REGISTRY).decode('utf-8')
        except Exception as e:
            logger.error("获取指标导出失败: %s", e)
            return ""


# 全局指标收集器实例
_UNIFIED_COLLECTOR: Optional[MetricsCollector] = None


def get_unified_collector() -> MetricsCollector:
    """获取统一的指标收集器实例"""
    global _UNIFIED_COLLECTOR
    if _UNIFIED_COLLECTOR is None:
        raise RuntimeError("统一的指标收集器未初始化。"
                           "请在运行器中初始化它。")
    return _UNIFIED_COLLECTOR


def init_unified_collector(node_type: str = "local", redis_client=None, node_id: str = "local",
                           coordinator=None, batch_size: int = 100, flush_interval: float = 1.0,
                           buffer_size: int = 10000) -> MetricsCollector:
    """初始化统一的指标收集器"""
    global _UNIFIED_COLLECTOR
    _UNIFIED_COLLECTOR = MetricsCollector(
        node_type,
        redis_client,
        node_id,
        coordinator,
        batch_size=batch_size,
        flush_interval=flush_interval,
        buffer_size=buffer_size
    )
    return _UNIFIED_COLLECTOR


def is_unified_collector_initialized() -> bool:
    """检查统一指标收集器是否已初始化"""
    global _UNIFIED_COLLECTOR
    return _UNIFIED_COLLECTOR is not None
